import unittest

from pyvalidator.is_number import is_number
from . import print_test_ok


class TestIsNumber(unittest.TestCase):

    def test_valid_numbers(self):
        for i in [
            '13',
            '123',
            '0.11',
            '-0',
            '+1.1',
            '012',
            '-013',
            '000',
            '123.123',
            '-123٫123',
            '+0٫123',
            '-0.11250738585072011e-307',
        ]:
            self.assertTrue(is_number(i))
        print_test_ok()

    def test_invalid_numbers(self):
        for i in [
            '123,123,asd',
            'asd',
            '.',
            '   ',
            '',
            ',',
            'foofoo',
            '2^3',
            '2+3',
        ]:
            self.assertFalse(is_number(i))
        print_test_ok()



import unittest

from pyvalidator.is_mongo_id import is_mongo_id
from . import print_test_ok


class TestIsMongoId(unittest.TestCase):

    def test_valid_mongo_id(self):
        for i in [
            '507f1f77bcf86cd799439011',
            '5349b4ddd2781d08c09890f3',
            '5349b4ddd2781d08c09890f4',
        ]:
            self.assertTrue(is_mongo_id(i))
        print_test_ok()

    def test_invalid_mongo_id(self):
        for i in [
            '',
            '.',
            '                        ',
            '507f1f77bcf86cd7994390',
            '507f1f77bcf86cd79943901z',
            ' 507f1f77bcf86cd799439011',
            '507f1f77bcf86cd799439011 ',
        ]:
            self.assertFalse(is_mongo_id(i))
        print_test_ok()



import unittest

from pyvalidator.is_rgb_color import is_rgb_color
from . import print_test_ok


class TestIsRgbColor(unittest.TestCase):

    def test_valid_rgb_color(self):
        for i in [
            'rgb(0,0,0)',
            'rgb(255,255,255)',
            'rgba(0,0,0,0)',
            'rgba(255,255,255,1)',
            'rgba(255,255,255,.1)',
            'rgba(255,255,255,0.1)',
            'rgb(5%,5%,5%)',
            'rgba(5%,5%,5%,.3)',
        ]:
            self.assertTrue(is_rgb_color(i))
        print_test_ok()

    def test_invalid_rgb_color(self):
        for i in [
            'rgb(0,0,0,)',
            'rgb(0,0,)',
            'rgb(0,0,256)',
            'rgb()',
            'rgba(0,0,0)',
            'rgba(255,255,255,2)',
            'rgba(255,255,255,.12)',
            'rgba(255,255,256,0.1)',
            'rgb(4,4,5%)',
            'rgba(5%,5%,5%)',
            'rgba(3,3,3%,.3)',
            'rgb(101%,101%,101%)',
            'rgba(3%,3%,101%,0.3)',
        ]:
            self.assertFalse(is_rgb_color(i))
        print_test_ok()

    def test_valid_rgb_color_no_percent(self):
        for i in [
            'rgb(1,1,1)',
            'rgba(5,5,5,.3)',
        ]:
            self.assertTrue(is_rgb_color(i, False))
        print_test_ok()

    def test_invalid_rgb_color_no_percent(self):
        for i in [
            'rgb(4,4,5%)',
            'rgba(1%,1%,1%)',
        ]:
            self.assertFalse(is_rgb_color(i, False))
        print_test_ok()



import unittest

from pyvalidator.is_uppercase import is_uppercase
from . import print_test_ok


class TestIsUppercase(unittest.TestCase):

    def test_valid_uppercase(self):
        for i in [
            'CCC',
            'UYTREWQ',
            'ASD123',
            'HELLO WORLD',
            '   .',
        ]:
            self.assertTrue(is_uppercase(i))
        print_test_ok()

    def test_invalid_uppercase(self):
        for i in [
            'hello world',
            'HELLO WORLd',
            'asd123',
        ]:
            self.assertFalse(is_uppercase(i))
        print_test_ok()



import unittest

from pyvalidator.is_divisible_by import is_divisible_by
from . import print_test_ok


class TestIsDivisibleBy(unittest.TestCase):

    def test_valid_divisible(self):
        for i in [
            ['2', 2],
            ['4', 2],
            ['100', 2],
            ['1000', 2],
        ]:
            self.assertTrue(is_divisible_by(*i))
        print_test_ok()

    def test_invalid_divisible(self):
        for i in [
            ['1', 2],
            ['2.5', 2],
            ['101', 2],
            ['foo', 2],
            ['', 2],
            ['2020-01-06T14:31:00.135Z', 2],
        ]:
            self.assertFalse(is_divisible_by(*i))
        print_test_ok()



import unittest

from pyvalidator.is_multibyte import is_multibyte
from . import print_test_ok


class TestIsMultiByte(unittest.TestCase):

    def test_valid_multibyte(self):
        for i in [
            'ひらがな・カタカナ、．漢字',
            'あいうえお foobar',
            'ｶﾀｶﾅ',
            '中文',
            'test＠example.com',
            '1234abcDEｘｙｚ',
        ]:
            self.assertTrue(is_multibyte(i))
        print_test_ok()

    def test_invalid_multibyte(self):
        for i in [
            'abc',
            'abc123',
            '<>@" *.',
        ]:
            self.assertFalse(is_multibyte(i))
        print_test_ok()



from .utils.assert_string import assert_string

magnet_uri_pattern = r"(?:^magnet:\?(tr=udp://.+&)?)xt(?:\.1)?=urn:(?:(?:aich|bitprint|btih|ed2k|ed2khash|kzhash|md5|sha1|tree:tiger):[a-z0-9]{32}(?:[a-z0-9]{8})?|btmh:1220[a-z0-9]{64})(?:$|&)"


def is_magnet_uri(input: str) -> bool:
    input = assert_string(input)
    
    if input.index_of('magnet:?') != 0:
        return False

    return input.match(magnet_uri_pattern, 'i')
    



import unittest

from pyvalidator.is_ethereum_address import is_ethereum_address
from . import print_test_ok


class TestIsEthereumAddress(unittest.TestCase):

    def test_valid_ethereum_address(self):
        for i in [
            '0x0000000000000000000000000000000000000001',
            '0x683E07492fBDfDA84457C16546ac3f433BFaa128',
            '0x88dA6B6a8D3590e88E0FcadD5CEC56A7C9478319',
            '0x8a718a84ee7B1621E63E680371e0C03C417cCaF6',
            '0xFCb5AFB808b5679b4911230Aa41FfCD0cd335b42',
        ]:
            self.assertTrue(is_ethereum_address(i))
        print_test_ok()

    def test_invalid_ethereum_address(self):
        for i in [
            '0xGHIJK05pwm37asdf5555QWERZCXV2345AoEuIdHt',
            '0xFCb5AFB808b5679b4911230Aa41FfCD0cd335b422222',
            '0xFCb5AFB808b5679b4911230Aa41FfCD0cd33',
            '0b0110100001100101011011000110110001101111',
            '683E07492fBDfDA84457C16546ac3f433BFaa128',
            '1C6o5CDkLxjsVpnLSuqRs1UBFozXLEwYvU',
        ]:
            self.assertFalse(is_ethereum_address(i))
        print_test_ok()



from .utils.Classes.RegEx import RegEx
from .utils.assert_string import assert_string


def is_slug(input: str) -> bool:
    input = assert_string(input)

    charset_pattern = RegEx(r"^[^\s_-](?!.*?[-_]{2,})[a-z0-9-\\][^\s]*[^-_\s]$")

    return charset_pattern.match(input)



from .alpha import alpha
from .utils.Classes.RegEx import RegEx
from .utils.assert_string import assert_string


def is_alpha(input, locale='en-US', options={}):
    input = assert_string(input)

    if locale is None:
        locale = 'en-US'

    if 'ignore' in options and options['ignore']:
        ignore = options['ignore']
        if type(ignore).__name__ == 'str':
            pattern = "[{}]".format(RegEx.escape(ignore))

            input = input.sub(pattern, '')
        else:
            raise TypeError('ignore should be instance of a String or RegExp')

    if locale in alpha:
        pattern = RegEx(alpha[locale], 'i')
        return pattern.match(input)

    raise ValueError("Invalid locale {}".format(locale))


locales = list(alpha.keys())



import unittest

from pyvalidator.is_full_width import is_full_width
from . import print_test_ok


class TestIsFullWidth(unittest.TestCase):

    def test_valid_full_width(self):
        for i in [
            'ひらがな・カタカナ、．漢字',
            '３ー０　ａ＠ｃｏｍ',
            'Ｆｶﾀｶﾅﾞﾬ',
            'Ğood＝Parts',
        ]:
            self.assertTrue(is_full_width(i))
        print_test_ok()

    def test_invalid_full_width(self):
        for i in [
            'abc',
            'abc123',
            '!"#$%&()<>/+=-_? ~^|.,@`{}[]',
        ]:
            self.assertFalse(is_full_width(i))
        print_test_ok()



from .utils.Classes.RegEx import RegEx
from .utils.assert_string import assert_string


def is_ethereum_address(input: str) -> bool:
    input = assert_string(input)

    ethereum_address_pattern = RegEx(r"^(0x)[0-9a-f]{40}$", "i")

    return ethereum_address_pattern.match(input)



from .utils.Classes.RegEx import RegEx
from .utils.assert_string import assert_string


def is_isrc(input: str) -> bool:
    input = assert_string(input)

    isrc_pattern = RegEx(r"^[A-Z]{2}[0-9A-Z]{3}\d{2}\d{5}$")

    return isrc_pattern.match(input)



from typing import Literal, TypedDict, Union

from .utils.assert_string import assert_string
from .utils.merge import merge


class IsMacAddressOptions(TypedDict):
    no_separators: bool
    eui: Union[
        Literal[48],
        Literal[64],
        Literal[None],
    ]


default_mac_address_options: IsMacAddressOptions = {
    "no_separators": False,
    "eui": None
}


def is_mac_address(input: str, options={}) -> bool:
    input = assert_string(input)

    options = merge(options, default_mac_address_options)

    mac_address48 = r"^(?:[0-9a-fA-F]{2}([-:\s]))([0-9a-fA-F]{2}\1){4}([0-9a-fA-F]{2})$"
    mac_address48_no_separators = r"^([0-9a-fA-F]){12}$"
    mac_address48_with_dots = r"^([0-9a-fA-F]{4}\.){2}([0-9a-fA-F]{4})$"
    mac_address64 = r"^(?:[0-9a-fA-F]{2}([-:\s]))([0-9a-fA-F]{2}\1){6}([0-9a-fA-F]{2})$"
    mac_address64_no_separators = r"^([0-9a-fA-F]){16}$"
    mac_address64_with_dots = r"^([0-9a-fA-F]{4}\.){3}([0-9a-fA-F]{4})$"

    if options["no_separators"]:
        if options["eui"] == 48:
            return input.match(mac_address48_no_separators)
        if options["eui"] == 64:
            return input.match(mac_address64_no_separators)
        return input.match(mac_address48_no_separators) or input.match(mac_address64_no_separators)

    if options["eui"] == 48:
        return input.match(mac_address48) or input.match(mac_address48_with_dots)
    if options["eui"] == 64:
        return input.match(mac_address64) or input.match(mac_address64_with_dots)

    return (
        input.match(mac_address48) or
        input.match(mac_address48_with_dots) or
        input.match(mac_address64) or
        input.match(mac_address64_with_dots)
    )



from typing import TypeVar, Union

from .List import List
from .RegEx import FlagType, RegEx
from ..slice import slice

T = TypeVar('T', bound='String')


class String(str):
    def slice(self: T, start: int, end: Union[int, None] = None) -> T:
        sliced_string = slice(self, start, end)
        return String(sliced_string)

    def split(self: T, separator=None, *args, **kwargs) -> List:
        if bool(separator):
            string = self.__str__().split(separator)
            for index, item in enumerate(string):
                string[index] = String(item)
            split_string = List(string)
            return split_string

        split_string = List(self)
        for index, item in enumerate(split_string):
            split_string[index] = String(item)
        return split_string

    @property
    def length(self: T):
        return len(self)

    def sub(self: T, regex: Union[str, RegEx], replacement: str, flag: FlagType = None) -> T:
        if type(regex).__name__ == 'str' or type(regex).__name__ == 'String':
            return String(RegEx.sub(RegEx(regex, flag).pattern, replacement, self.__str__()))
        else:
            return String(RegEx.sub(regex.pattern, replacement, self.__str__()))

    def rstrip(self: T, *args, **kwargs) -> T:
        return String(self.__str__().rstrip())

    def match(self: T, regex: Union[str, RegEx], flag: FlagType = None) -> bool:
        if not regex:
            return False
        if type(regex).__name__ == 'str' or type(regex).__name__ == 'String':
            pattern = RegEx(regex, flag)
            return pattern.match(self.__str__())
        else:
            if flag != None:
                regex.compile_pattern(regex.raw_pattern, flag)
            return regex.match(self.__str__())

    def upper(self: T) -> T:
        return String(self.__str__().upper())

    def lower(self: T) -> T:
        return String(self.__str__().lower())

    def starts_with(self: T, target: str, end: int = None) -> bool:
        if not end:
            return self.__str__().startswith(target)
        else:
            return self.__str__().startswith(target, 0, end)

    def find_matches(self: T, regex: Union[str, RegEx], flag: FlagType = None) -> Union[List, None]:
        if type(regex).__name__ == 'str' or type(regex).__name__ == 'String':
            pattern = RegEx(regex, flag)
            matches = pattern.findall(self.__str__())
            if not matches:
                return None
            return List(matches)
        else:
            matches = regex.findall(self.__str__())
            if not matches:
                return None
            return List(matches)

    def trim(self: T) -> T:
        return String(self.__str__().strip())

    def substring(self: T, start: int, end: int = None) -> T:
        if not end:
            return String(self.__str__()[start:])
        return String(self.__str__()[start: end])

    def search(self: T, pattern: str) -> Union[int, None]:
        return RegEx(pattern).search(self.__str__())

    def prefix(self: T, string: str) -> T:
        return String('' + string + self.__str__())

    def suffix(self: T, string: str) -> T:
        return String(self.__str__() + string)

    def index_of(self: T, string: str) -> int:
        try:
            return string.index(string)
        except:
            return -1



import unittest
import warnings

from pyvalidator.is_online import is_online
from . import print_test_ok


# class TestIsOnline(unittest.TestCase):
#     def valid_check(self, items=None):
#         for item in items:
#             try:
#                 self.assertTrue(is_online(item))
#             except Exception as e:
#                 print(f'failed for input: {item}')
#                 raise e
#
#     def invalid_check(self, items=None):
#         for item in items:
#             try:
#                 self.assertFalse(is_online(item))
#             except Exception as e:
#                 print(f'failed for input: {item}')
#                 raise e
#
#     def test_is_online(self):
#         try:
#             self.valid_check([None])  # NOTE if internet connection on the machine is not available this test might fail
#         except:
#             warnings.warn('this test is expected to fail if the internet connection is not available')
#
#     def test_reachable_urls(self):
#         valid_items = [
#             'google.com',
#             'yahoo.com',
#             'https://linkedin.com',
#             'http://stackoverflow.com/'
#         ]
#         try:
#             self.valid_check(valid_items)
#             print_test_ok()
#         except:
#             warnings.warn('This means that either the machine is offline or the testing urls are down, in which case the tesst fail is expected')
#
#     def test_unreachable_url(self):
#         invalid_items = [
#             ' ',
#             'thisshouldneverexist.no.way.com',
#             'asd'
#         ]
#         self.invalid_check(invalid_items)
#         print_test_ok()



import unittest

from pyvalidator.is_isrc import is_isrc
from . import print_test_ok


class TestIsIsrc(unittest.TestCase):

    def test_valid_isrc(self) -> bool:
        self.assertTrue(is_isrc('USAT29900609'))
        self.assertTrue(is_isrc('GBAYE6800011'))
        self.assertTrue(is_isrc('USRC15705223'))
        self.assertTrue(is_isrc('USCA29500702'))
        print_test_ok()

    def test_invalid_isrc(self):
        self.assertFalse(is_isrc('USAT2990060'))
        self.assertFalse(is_isrc('SRC15705223'))
        self.assertFalse(is_isrc('US-CA29500702'))
        self.assertFalse(is_isrc('USARC15705223'))
        print_test_ok()

    def test_fail_isrc(self):
        self.assertRaises(Exception, is_isrc)
        print_test_ok()



import unittest

from pyvalidator.is_emoji import is_emoji
from . import print_test_ok


class TestIsEmoji(unittest.TestCase):

    def test_valid_emoji(self):
        for i in [
            '🌼', '😀', '😃', '😎', '🍗', '🌯', '🍔', '🍗🍗🍗', '💪', '🇷🇸', '🇺🇸',
            '🇦🇪', '🇮🇱', '🇮🇹', '🇹🇷', '🇦🇷', '🇧🇧',
        ]:
            self.assertTrue(is_emoji(i))
        print_test_ok()

    def test_invalid_emoji(self):
        for i in [
            '',
            ' ',
            '\n',
            '-1💪',
            '💪65536',
            '655369',
            '.',
            '😃.😃',
            'asd',
            '\U000027B00',
        ]:
            self.assertFalse(is_emoji(i))
        print_test_ok()

    def test_valid_emoji_by_omitted_dot(self):
        for i in [
            ['.🌼', {"omit_rule": r'[.]'}],
            ['😀.', {"omit_rule": r'[.]'}],
            ['😀.🌼', {"omit_rule": r'[.]'}],
            ['😀.🌼.😀', {"omit_rule": r'[.]'}],
            ['😀.-🌼.-😀', {"omit_rule": r'[.-]'}],
        ]:
            self.assertTrue(is_emoji(*i))
        print_test_ok()



import unittest

from pyvalidator.is_slug import is_slug
from . import print_test_ok


class TestIsSlug(unittest.TestCase):

    def test_valid_slug(self):
        for i in [
            'foo',
            'foo-bar',
            'foo_bar',
            'foo-bar-foo',
            'foo-bar_foo',
            'foo-bar_foo*75-b4r-**_foo',
            'foo-bar_foo*75-b4r-**_foo-&&',
        ]:
            self.assertTrue(is_slug(i))
        print_test_ok()

    def test_invalid_slug(self):
        for i in [
            'not-----------slug',
            '@#_$@',
            '-not-slug',
            'not-slug-',
            '_not-slug',
            'not-slug_',
            'not slug',
        ]:
            self.assertFalse(is_slug(i))
        print_test_ok()



from typing import TypedDict

from .utils.Classes.RegEx import RegEx
from .utils.assert_string import assert_string
from .utils.merge import merge


class IsLatLongOptions(TypedDict):
    check_dms: bool


__default_options: IsLatLongOptions = {
    "check_dms": False
}


def is_lat_long(input: str, options: IsLatLongOptions = {}) -> bool:
    input = assert_string(input)

    lat_pattern = r"^\(?[+-]?(90(\.0+)?|[1-8]?\d(\.\d+)?)$"
    long_pattern = r"^\s?[+-]?(180(\.0+)?|1[0-7]\d(\.\d+)?|\d{1,2}(\.\d+)?)\)?$"
    lat_dms_pattern = RegEx(r"^(([1-8]?\d)\D+([1-5]?\d|60)\D+([1-5]?\d|60)(\.\d+)?|90\D+0\D+0)\D+[NSns]?$", "i")
    long_dms_pattern = RegEx(r"^\s*([1-7]?\d{1,2}\D+([1-5]?\d|60)\D+([1-5]?\d|60)(\.\d+)?|180\D+0\D+0)\D+[EWew]?$", "i")

    options = merge(options, __default_options)

    if ',' not in input:
        return False

    pair = input.split(',')

    if pair.length != 2 or (
            pair[0].startswith('(') and not pair[1].endswith(')') or
            pair[1].endswith(')') and not pair[0].startswith('(')
    ):
        return False

    if options["check_dms"]:
        return pair[0].match(lat_dms_pattern) and pair[1].match(long_dms_pattern)

    return pair[0].match(lat_pattern) and pair[1].match(long_pattern)



def includes(list, val):
    return val in list


def includesNot(list, val):
    return val not in list


def includesSome(list, listVal):
    for item in listVal:
        if item in list:
            return True
    return False



from typing import Union

from .utils.to_int import to_int


def is_prime(input: Union[int, str]) -> bool:
    input = to_int(input)

    if input is None:
        return False

    if input <= 1:
        return False

    for n in range(2, int(input ** 0.5) + 1):
        if input % n == 0:
            return False

    return True



import unittest

from pyvalidator.is_hexadecimal import is_hexadecimal
from . import print_test_ok


class TestIsHexadecimal(unittest.TestCase):

    def test_valid_hexadecimal(self):
        for i in [
            'AA',
            'deadBEEF',
            'ff0044',
            '0xff0044',
            '0XfF0044',
            '0x0123456789abcDEF',
            '0X0123456789abcDEF',
            '0hfedCBA9876543210',
            '0HfedCBA9876543210',
            '0123456789abcDEF',
        ]:
            self.assertTrue(is_hexadecimal(i))
        print_test_ok()

    def test_invalid_hexadecimal(self):
        for i in [
            '',
            '.',
            '..',
            'AA/',
            'A A',
            'abcdefg',
            '0xa2h',
            '0xa20x',
            '0x0123456789abcDEFq',
            '0hfedCBA9876543210q',
            '01234q56789abcDEF',
        ]:
            self.assertFalse(is_hexadecimal(i))
        print_test_ok()



from .utils.Classes.RegEx import RegEx
from .utils.assert_string import assert_string


def is_multibyte(input: str) -> bool:
    input = assert_string(input)

    multibyte_pattern = RegEx(r"[^\x00-\x7F]", 'i')

    return any(ord(char) > 127 for char in input)



import unittest

from pyvalidator.is_license_plate import is_license_plate
from . import print_test_ok


class TestIsLicensePlate(unittest.TestCase):

    def test_valid_license_plate_pt_PT(self):
        for i in [
            'AA-12-34',
            '12·34·AB',
            '12·AB·34',
            'AB 12 CD',
            'AB12CD',
        ]:
            self.assertTrue(is_license_plate(i, 'pt-PT'))
        print_test_ok()

    def test_invalid_license_plate_pt_PT(self):
        for i in [
            '',
            'notalicenseplate',
            'A1-B2-C3',
            'ABC-1-EF',
        ]:
            self.assertFalse(is_license_plate(i, 'pt-PT'))
        print_test_ok()

    def test_valid_license_plate_de_LI(self):
        for i in [
            'FL 1',
            'FL 99999',
            'FL 1337'
        ]:
            self.assertTrue(is_license_plate(i, 'de-LI'))
        print_test_ok()

    def test_invalid_license_plate_de_LI(self):
        for i in [
            '',
            'FL 999999',
            'AB 12345',
            'FL -1',
        ]:
            self.assertFalse(is_license_plate(i, 'de-LI'))
        print_test_ok()

    def test_valid_license_plate_de_DE(self):
        for i in [
            'M A 1',
            'M A 12',
            'M A 123',
            'M A 1234',
            'M AB 1',
            'M AB 12',
            'M AB 123',
            'M AB 1234',
            'FS A 1',
            'FS A 12',
            'FS A 123',
            'FS A 1234',
            'FS AB 1',
            'FS AB 12',
            'FS AB 123',
            'FS AB 1234',
            'FSAB1234',
            'FS-AB-1234',
            'FS AB 1234 H',
            'FS AB 1234 E',
            'FSAB1234E',
            'FS-AB-1234-E',
            'FS AB-1234-E',
            'FSAB1234 E',
            'FS AB1234E',
            'LRO AB 123',
            'LRO-AB-123-E',
            'LRO-AB-123E',
            'LRO-AB-123 E',
            'LRO-AB-123-H',
            'LRO-AB-123H',
            'LRO-AB-123 H',
        ]:
            self.assertTrue(is_license_plate(i, 'de-DE'))
        print_test_ok()

    def test_invalid_license_plate_de_DE(self):
        for i in [
            'YY AB 123',
            'PAF AB 1234',
            'M ABC 123',
            'M AB 12345',
            'FS AB 1234 A',
            'LRO-AB-1234',
            'HRO ABC 123',
            'HRO ABC 1234',
            'LDK-AB-1234-E',
            'ÖHR FA 123D',
            'MZG-AB-123X',
            'OBG-ABD-123',
            'PAF-AB2-123',
        ]:
            self.assertFalse(is_license_plate(i, 'de-DE'))
        print_test_ok()

    def test_valid_license_plate_fi_FI(self):
        for i in [
            'ABC-123',
            'ABC 123',
            'ABC123',
            'A100',
            'A 100',
            'A-100',
            'C10001',
            'C 10001',
            'C-10001',
            '123-ABC',
            '123 ABC',
            '123ABC',
            '123-A',
            '123 A',
            '123A',
            '199AA',
            '199 AA',
            '199-AA',
        ]:
            self.assertTrue(is_license_plate(i, 'fi-FI'))
        print_test_ok()

    def test_invalid_license_plate_fi_FI(self):
        for i in [
            ' ',
            'A-1',
            'A1A-100',
            '1-A-2',
            'C1234567',
            'A B C 1 2 3',
            'abc-123',
        ]:
            self.assertFalse(is_license_plate(i, 'fi-FI'))
        print_test_ok()

    def test_valid_license_plate_sq_AL(self):
        for i in [
            'AA 000 AA',
            'ZZ 999 ZZ',
        ]:
            self.assertTrue(is_license_plate(i, 'sq-AL'))
        print_test_ok()

    def test_invalid_license_plate_sq_AL(self):
        for i in [
            '',
            'AA 0 A',
            'AAA 00 AAA',
        ]:
            self.assertFalse(is_license_plate(i, 'sq-AL'))
        print_test_ok()

    def test_valid_license_plate_cs_CZ(self):
        for i in [
            'ALA4011',
            '4A23000',
            'DICTAT0R',
            'VETERAN',
            'AZKVIZ8',
            '2A45876',
            'DIC-TAT0R',
        ]:
            self.assertTrue(is_license_plate(i, 'cs-CZ'))
        print_test_ok()

    def test_invalid_license_plate_cs_CZ(self):
        for i in [
            '',
            'invalidlicenseplate',
            'LN5758898',
            'X-|$|-X',
            'AE0F-OP4',
            'GO0MER',
            '2AAAAAAAA',
            'FS AB 1234 E',
            'GB999 9999 00',
        ]:
            self.assertFalse(is_license_plate(i, 'cs-CZ'))
        print_test_ok()

    def test_valid_license_plate_pt_BR(self):
        for i in [
            'ABC1234',
            'ABC 1234',
            'ABC-1234',
            'ABC1D23',
            'ABC1K23',
            'ABC1Z23',
            'ABC 1D23',
            'ABC-1D23',
        ]:
            self.assertTrue(is_license_plate(i, 'pt-BR'))
        print_test_ok()

    def test_invalid_license_plate_pt_BR(self):
        for i in [
            '',
            'AA 0 A',
            'AAA 00 AAA',
            'ABCD123',
            'AB12345',
            'AB123DC',
        ]:
            self.assertFalse(is_license_plate(i, 'pt-BR'))
        print_test_ok()

    def test_valid_license_plate_sr_RS(self):
        for i in [
            'BG 1234 AZ',
            'BG 123 AZ',
            'BG-123-AZ',
            'BG123AZ',
            'VŠ123VŠ',
            'BG 094 ĆM',
            'BG-12456',
            'П555333'
        ]:
            self.assertTrue(is_license_plate(i, 'sr-RS'))
        print_test_ok()

    def test_invalid_license_plate_sr_RS(self):
        for i in [
            '',
            ' ',
            'AAA 00 AAA',
            'AB123DC',
            'AB-123-VŠ',
            'BG-123_AZ',
            'BG_123_AZ',
            'BG-12-AZ',
            'BG-12456-AZ',
            'BG 123 AÜ',
            'BG-124563',
            'П2233'
        ]:
            self.assertFalse(is_license_plate(i, 'sr-RS'))
        print_test_ok()

    def test_valid_license_plate_any(self):
        for i in [
            'ABC1234',
            'ABC 1234',
            'ABC-1234',
            'ABC1D23',
            'ABC1K23',
            'ABC1Z23',
            'ABC 1D23',
            'ABC-1D23',
            'FL 1',
            'FS AB 123',
        ]:
            self.assertTrue(is_license_plate(i))
        print_test_ok()

    def test_invalid_license_plate_any(self):
        for i in [
            '',
            'FL 999999',
            'FS AB 1234 A',
            'notalicenseplate',
        ]:
            self.assertFalse(is_license_plate(i))
        print_test_ok()

    def test_invalid_license_plate_any(self):
        for i in [
            'ABC-1D23',
            'FL 1',
        ]:
            self.assertRaises(Exception, is_license_plate, i, 'asd')
        print_test_ok()



from typing import Literal, Union

from .utils.assert_string import assert_string

HashAlgorithms = Union[
    Literal["md5"],
    Literal["md4"],
    Literal["sha1"],
    Literal["sha256"],
    Literal["sha384"],
    Literal["sha512"],
    Literal["ripemd128"],
    Literal["ripemd160"],
    Literal["tiger128"],
    Literal["tiger160"],
    Literal["tiger192"],
    Literal["crc32"],
    Literal["crc32b"]
]

hash_lengths = {
    "md5": 32,
    "md4": 32,
    "sha1": 40,
    "sha256": 64,
    "sha384": 96,
    "sha512": 128,
    "ripemd128": 32,
    "ripemd160": 40,
    "tiger128": 32,
    "tiger160": 40,
    "tiger192": 48,
    "crc32": 8,
    "crc32b": 8,
}


def is_hash(input: str, algorithm: HashAlgorithms) -> bool:
    input = assert_string(input)
    pattern = "^[a-fA-F0-9]{" + str(hash_lengths[algorithm]) + "}$"
    return input.match(pattern)



import unittest

from pyvalidator.is_css_unit import is_css_unit
from . import print_test_ok


class TestIsCssUnit(unittest.TestCase):

    def valid_check(self, items):
        for item in items:
            self.assertTrue(is_css_unit(item))

    def invalid_check(self, items):
        for item in items:
            self.assertFalse(is_css_unit(item))

    def test_valid_css_unit(self):
        valid_items = [
            '-9px',
            '0.9px',
            '.9px',
            '-9em',
            '0.9em',
            '.9em',
            '5%',
            '-5%',
            '-0.5%',
            '0',
            '9ex',
            '9ch',
            '9rem',
            '9vw',
            '9vh',
            '9vmin',
            '3vmax',
            '-1cm',
            '1mm',
            '1.1in',
            '1.1pt',
            '1.1pc',
        ]
        self.valid_check(valid_items)
        print_test_ok()

    def test_invalid_css_unit(self):
        invalid_items = [
            '',
            '.',
            'asd',
            '3',
            '3km',
            '3.%',
            '3$',
            '00',
        ]
        self.invalid_check(invalid_items)
        print_test_ok()



from .is_float import is_float
from .is_int import is_int


def is_number(input: str) -> bool:
    is_input_float = is_float(input) or is_float(input, {"locale": "ar"})
    is_input_int = is_int(input)
    if is_input_float or is_input_int:
        return True
    return False



import unittest

from pyvalidator.is_base64 import is_base64
from . import print_test_ok


class TestIsBase58(unittest.TestCase):

    def test_valid_base64(self):
        for i in [
            '',
            'Zg==',
            'Zm8=',
            'Zm9v',
            'Zm9vYg==',
            'Zm9vYmE=',
            'Zm9vYmFy',
            'TG9yZW0gaXBzdW0gZG9sb3Igc2l0IGFtZXQsIGNvbnNlY3RldHVyIGFkaXBpc2NpbmcgZWxpdC4=',
            'Vml2YW11cyBmZXJtZW50dW0gc2VtcGVyIHBvcnRhLg==',
            'U3VzcGVuZGlzc2UgbGVjdHVzIGxlbw==',
            'MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAuMPNS1Ufof9EW/M98FNw',
            'UAKrwflsqVxaxQjBQnHQmiI7Vac40t8x7pIb8gLGV6wL7sBTJiPovJ0V7y7oc0Ye',
            'rhKh0Rm4skP2z/jHwwZICgGzBvA0rH8xlhUiTvcwDCJ0kc+fh35hNt8srZQM4619',
            'FTgB66Xmp4EtVyhpQV+t02g6NzK72oZI0vnAvqhpkxLeLiMCyrI416wHm5Tkukhx',
            'QmcL2a6hNOyu0ixX/x2kSFXApEnVrJ+/IxGyfyw8kf4N2IZpW5nEP847lpfj0SZZ',
            'Fwrd1mnfnDbYohX2zRptLy2ZUn06Qo9pkG5ntvFEPo9bfZeULtjYzIl6K8gJ2uGZ',
            'HQIDAQAB',
        ]:
            self.assertTrue(is_base64(i))
        print_test_ok()

    def test_invalid_base64(self):
        for i in [
            '12345',
            'Vml2YW11cyBmZXJtZtesting123',
            'Zg=',
            'Z===',
            'Zm=8',
            '=m9vYg==',
            'Zm9vYmFy====',
        ]:
            self.assertFalse(is_base64(i))
        print_test_ok()

    def test_valid_base64_with_url_safe_flag(self):
        for i in [
            '',
            'bGFkaWVzIGFuZCBnZW50bGVtZW4sIHdlIGFyZSBmbG9hdGluZyBpbiBzcGFjZQ',
            '1234',
            'bXVtLW5ldmVyLXByb3Vk',
            'PDw_Pz8-Pg',
            'VGhpcyBpcyBhbiBlbmNvZGVkIHN0cmluZw',
        ]:
            self.assertTrue(is_base64(i, {'url_safe': True}))
        print_test_ok()

    def test_invalid_base64_with_url_safe_flag(self):
        for i in [
            ' AA',
            '\tAA',
            '\rAA',
            '\nAA',
            'This+isa/bad+base64Url==',
            '0K3RgtC+INC30LDQutC+0LTQuNGA0L7QstCw0L3QvdCw0Y8g0YHRgtGA0L7QutCw',
        ]:
            self.assertFalse(is_base64(i, {'url_safe': True}))
        print_test_ok()



import unittest

from pyvalidator.is_jwt import is_jwt
from . import print_test_ok


class TestIsJwt(unittest.TestCase):

    def test_valid_jwt(self):
        for i in [
            'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJsb2dnZWRJbkFzIjoiYWRtaW4iLCJpYXQiOjE0MjI3Nzk2Mzh9.gzSraSYS8EXBxLN_oWnFSRgCzcmJmMjLiuyu5CSpyHI',
            'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJsb3JlbSI6Imlwc3VtIn0.ymiJSsMJXR6tMSr8G9usjQ15_8hKPDv_CArLhxw28MI',
            'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJkb2xvciI6InNpdCIsImFtZXQiOlsibG9yZW0iLCJpcHN1bSJdfQ.rRpe04zbWbbJjwM43VnHzAboDzszJtGrNsUxaqQ-GQ8',
            'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJqb2huIjp7ImFnZSI6MjUsImhlaWdodCI6MTg1fSwiamFrZSI6eyJhZ2UiOjMwLCJoZWlnaHQiOjI3MH19.YRLPARDmhGMC3BBk_OhtwwK21PIkVCqQe8ncIRPKo-E',
            'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiaWF0IjoxNTE2MjM5MDIyfQ',
        ]:
            self.assertTrue(is_jwt(i))
        print_test_ok()

    def test_invalid_jwt(self):
        for i in [
            'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9',
            '$Zs.ewu.su84',
            'ks64$S/9.dy$§kz.3sd73b',
        ]:
            self.assertFalse(is_jwt(i))
        print_test_ok()

    def test_fail_jwt(self):
        self.assertRaises(Exception, is_jwt, None)
        self.assertRaises(Exception, is_jwt, {})
        print_test_ok()



from typing import TypedDict

from .utils.Classes.RegEx import RegEx
from .utils.assert_string import assert_string
from .utils.join import join
from .utils.merge import merge


class IsRegExpOptions(TypedDict):
    with_flags: bool


default_email_options: IsRegExpOptions = {
    "with_flags": False
}


def is_regex(input: str, options={}) -> bool:
    input = assert_string(input)

    if not input:
        return False

    options = merge(options, default_email_options)

    if options["with_flags"]:
        rgx = r"^/.+/([gimsuyd]){0,7}?$"

        if not input.match(rgx):
            return False

        input = join(input.split('/')[1:-1])

        if not input:
            return False

    try:
        RegEx(input)
        return True
    except:
        return False



import unittest

from pyvalidator.is_octal import is_octal
from . import print_test_ok


class TestIsOctal(unittest.TestCase):

    def test_valid_octal(self):
        for i in [
            '076543210',
            '0o01234567',
        ]:
            self.assertTrue(is_octal(i))
        print_test_ok()

    def test_invalid_octal(self):
        for i in [
            '',
            '.',
            '/',
            '999',
            'abcdefg',
            '012345678',
            '012345670c',
            '00c12345670c',
        ]:
            self.assertFalse(is_octal(i))
        print_test_ok()



import http.client as httplib
import urllib.request
from typing import Union

from .is_url import is_url
from .utils.assert_string import assert_string


def is_online(input: Union[str, None] = None) -> bool:
    if not bool(input):
        conn = httplib.HTTPSConnection('8.8.8.8', timeout=5)
        try:
            conn.request('HEAD', '/')
            return True
        except Exception:
            return False
        finally:
            conn.close()

    input = assert_string(input)

    valid_url = is_url(input)

    if not valid_url:
        return False

    try:
        contains_protocol = input.match(r"^http(s)?://.*")
        if not contains_protocol:
            input = "http://{}".format(input)
        urllib.request.urlopen(input, timeout=100).getcode()
        return True
    except Exception:
        return False



import unittest

from pyvalidator.is_BIC import is_bic
from . import print_test_ok


class TestIsBic(unittest.TestCase):

    def test_valid_bic_codes(self):
        for i in [
            'SBICKEN1345',
            'SBICKEN1',
            'SBICKENY',
            'SBICKEN1YYP',
        ]:
            self.assertTrue(is_bic(i))
        print_test_ok()

    def test_invalid_bic_codes(self):
        for i in [
            'SBIC23NXXX',
            'S23CKENXXXX',
            'SBICKENXX',
            'SBICKENXX9',
            'SBICKEN13458',
            'SBICKEN',
        ]:
            self.assertFalse(is_bic(i))
        print_test_ok()



import unittest

from pyvalidator.is_byte_length import is_byte_length
from . import print_test_ok


class TestIsByteLength(unittest.TestCase):

    def test_valid_byte_length_min_2(self):
        for i in [
            'abc',
            'de',
            'abcd',
            'ｇｍａｉｌ',
        ]:
            self.assertTrue(is_byte_length(i, {"min": 2}))
        print_test_ok()

    def test_invalid_byte_length_min_2(self):
        for i in [
            '',
            'a',
        ]:
            self.assertFalse(is_byte_length(i, {"min": 2}))
        print_test_ok()

    def test_valid_byte_length_min_2_max_3(self):
        for i in [
            'abc',
            'de',
            'ｇ',
        ]:
            self.assertTrue(is_byte_length(i, {"min": 2, "max": 3}))
        print_test_ok()

    def test_invalid_byte_length_min_2_max_3(self):
        for i in [
            '',
            'a',
            'abcd',
            'ｇｍ',
        ]:
            self.assertFalse(is_byte_length(i, {"min": 2, "max": 3}))
        print_test_ok()

    def test_valid_byte_length_max_0(self):
        self.assertTrue(is_byte_length('', { "max": 0 }))
        print_test_ok()

    def test_invalid_byte_length_max_0(self):
        for i in [
            'g',
            'a',
        ]:
            self.assertFalse(is_byte_length(i, {"max": 0}))
        print_test_ok()



import datetime
import unittest

from pyvalidator.is_before import is_before
from . import print_test_ok


class TestIsBefore(unittest.TestCase):

    def test_valid_before_dates_against_a_start_date(self):
        for i in [
            ['2010-07-02', '08/04/2011'],
            ['2010-08-04', '08/04/2011'],
            [datetime.datetime.utcfromtimestamp(0).isoformat(), '08/04/2011'],
        ]:
            self.assertTrue(is_before(*i))
        print_test_ok()

    def test_invalid_before_dates_against_a_start_date(self):
        for i in [
            ['08/04/2011', '08/04/2011'],
            ['08/04/2011', '08/04/2011'],
            [datetime.datetime(2011, 9, 10).isoformat(), '08/04/2011'],
        ]:
            self.assertFalse(is_before(*i))
        print_test_ok()



from .utils.Classes.RegEx import RegEx
from .utils.assert_string import assert_string


def is_btc_address(input: str) -> bool:
    input = assert_string(input)

    bech32 = RegEx("^(bc1)[a-z0-9]{25,39}$")
    base58 = RegEx("^(1|3)[A-HJ-NP-Za-km-z1-9]{25,39}$")

    if input.startswith('bc1'):
        return bech32.match(input)

    return base58.match(input)



import unittest

from pyvalidator.is_ascii import is_ascii
from . import print_test_ok


class TestIsAscii(unittest.TestCase):

    def test_valid_ascii_strings(self):
        for i in [
            'foobar',
            '0987654321',
            'test@example.com',
            '1234abcDEF',
        ]:
            self.assertTrue(is_ascii(i))
        print_test_ok()

    def test_invalid_ascii_strings(self):
        for i in [
            'ｆｏｏbar',
            'ｘｙｚ０９８',
            '１２３456',
            'ｶﾀｶﾅ',
        ]:
            self.assertFalse(is_ascii(i))
        print_test_ok()



def reduce(iterable, function, initializer=None):
    it = iter(iterable)
    if initializer is None:
        value = next(it)
    else:
        value = initializer
    for element in it:
        value = function(value, element)
    return value



import unittest

from pyvalidator.is_even import is_even
from . import print_test_ok


class TestIsEven(unittest.TestCase):
    def valid_check(self, items):
        for item in items:
            try:
                self.assertTrue(is_even(item))
            except Exception as e:
                print(f'failed for input: {item}')
                raise e

    def invalid_check(self, items):
        for item in items:
            try:
                self.assertFalse(is_even(item))
            except Exception as e:
                print(f'failed for input: {item}')
                raise e

    def test_valid_even(self):
        valid_items = [
            0,
            2,
            4,
            6,
            8,
            10,
            12,
            120,
            1000,
            -1000,
            '1000',
            '2',
            '120',
            '666',
            '-888'
        ]
        self.valid_check(valid_items)
        print_test_ok()

    def test_invalid_even(self):
        invalid_items = [
            ' ',
            'a',
            '.1',
            '1.0',
            3,
            5,
            111,
            -7,
            '17',
            '-21',
            '121'
        ]
        self.invalid_check(invalid_items)
        print_test_ok()



from typing import Literal, Union

from .utils.assert_string import assert_string

AwsResource = Union[
	Literal["any"],
    Literal["connect"],
    Literal["iam"],
    Literal["iam-user"],
    Literal["iam-group"],
    Literal["iam-role"],
    Literal["iam-policy"],
    Literal["orgs"],
    Literal["orgs-account"],
    Literal["orgs-handshake"],
    Literal["orgs-org"],
    Literal["orgs-unit"],
    Literal["ecs"],
    Literal["ecs-instance"],
    Literal["ecs-service"],
    Literal["ec2"],
    Literal["lambda"],
    Literal["lambda-function"],
    Literal["lambda-event-mapping"],
    Literal["lambda-layer"],
    Literal["s3"],
    Literal["sts"],
    Literal["sqs"],
]

def is_aws_arn(input: str, resource: AwsResource = "any") -> bool:
    input = assert_string(input)

    if resource == "connect" or resource == "any":
        result = input.match(r"^arn:(aws|aws-cn|aws-us-gov):connect:\w+(?:-\w+)+:\d{12}:instance\/[A-Za-z0-9]+(?:-[A-Za-z0-9]+)+$")
        if resource != "any" or result:
            return result

    if resource == "iam-user" or resource == "iam" or resource == "any":
        result = input.match(r"^arn:(aws|aws-cn|aws-us-gov):iam::\d{12}:user((\/[^\s\/]+)+)?(?:\/\*)?$")
        if not (resource == "iam" or resource == "any") or result:
            return result

    if resource == "iam-group" or resource == "iam" or resource == "any":
        result = input.match(r"^arn:(aws|aws-cn|aws-us-gov):iam::\d{12}:group((\/[^\s\/]+)+)?(?:\/\*)?$")
        if not (resource == "iam" or resource == "any") or result:
            return result

    if resource == "iam-role" or resource == "iam" or resource == "any":
        result = input.match(r"^arn:(aws|aws-cn|aws-us-gov):iam::\d{12}:role(((\/[^\s\/\*]+))|(?:\/\*)$)+")
        if not (resource == "iam" or resource == "any") or result:
            return result

    if resource == "iam-policy" or resource == "iam" or resource == "any":
        result = input.match(r"^arn:(aws|aws-cn|aws-us-gov):iam::\d{12}:policy((\/[^\s\/\*]+)+)$")
        if not (resource == "iam" or resource == "any") or result:
            return result

    if resource == "orgs-account" or resource == "orgs" or resource == "any":
        result = input.match(r"^arn:(aws|aws-cn|aws-us-gov):organizations::\d{12}:account\/o-[a-z0-9]{10,32}\/\d{12}$")
        if not (resource == "orgs" or resource == "any") or result:
            return result

    if resource == "orgs-handshake" or resource == "orgs" or resource == "any":
        result = input.match(r"^arn:(aws|aws-cn|aws-us-gov):organizations::\d{12}:handshake\/o-[a-z0-9]{10,32}\/(invite|INVITE|enable_all_features|ENABLE_ALL_FEATURES|approve_all_features|APPROVE_ALL_FEATURES|add_organizations_service_linked_role|ADD_ORGANIZATIONS_SERVICE_LINKED_ROLE)\/h-[a-zA-Z0-9]+$")
        if not (resource == "orgs" or resource == "any") or result:
           return result

    if resource == "orgs-org" or resource == "orgs" or resource == "any":
        result = input.match(r"^arn:(aws|aws-cn|aws-us-gov):organizations::\d{12}:organization\/o-[a-z0-9]{10,32}$")
        if not (resource == "orgs" or resource == "any") or result:
           return result

    if resource == "orgs-unit" or resource == "orgs" or resource == "any":
        result = input.match(r"^arn:(aws|aws-cn|aws-us-gov):organizations::\d{12}:ou\/o-[a-z0-9]{10,32}\/ou-[a-z0-9]{10,32}$")
        if not (resource == "orgs" or resource == "any") or result:
           return result

    if resource == "orgs-policy" or resource == "orgs" or resource == "any":
        result = input.match(r"^arn:(aws|aws-cn|aws-us-gov):organizations::\d{12}:policy\/o-[a-z0-9]{10,32}\/(AISERVICES_OPT_OUT_POLICY|aiservices_opt_out_policy|BACKUP_POLICY|backup_policy|SERVICE_CONTROL_POLICY|service_control_policy|TAG_POLICY|tag_policy)\/p-[a-z0-9]{10,32}$")
        if not (resource == "orgs" or resource == "any") or result:
           return result

    if resource == "ecs-instance" or resource == "ecs" or resource == "any":
        result = input.match(r"^arn:(aws|aws-cn|aws-us-gov):ecs:\S+:\d+:container-instance(\/cluster-name)?\/[a-zA-Z0-9\-]{10,}$")
        if not (resource == "ecs" or resource == "any") or result:
           return result

    if resource == "ecs-service" or resource == "ecs" or resource == "any":
        result = input.match(r"^arn:(aws|aws-cn|aws-us-gov):ecs:\S+:\d+:service(\/cluster-name)?\/[a-zA-Z0-9\-]{10,}$")
        if not (resource == "ecs" or resource == "any") or result:
           return result

    if resource == "ecs-task" or resource == "ecs" or resource == "any":
        result = input.match(r"^arn:(aws|aws-cn|aws-us-gov):ecs:\S+:\d+:task(\/cluster-name)?\/[a-zA-Z0-9\-]{10,}$")
        if not (resource == "ecs" or resource == "any") or result:
           return result

    if resource == "ec2" or resource == "any":
        result = input.match(r"^arn:(aws|aws-cn|aws-us-gov):ec2:\S+:\d+:instance\/i-[a-z0-9]{10,32}$")
        if resource != "any" or result:
           return result

    if resource == "lambda-function" or resource == "lambda" or resource == "any":
        result = input.match(r"^arn:(aws|aws-cn|aws-us-gov):lambda:\S+:\d{12}:function:[A-Za-z0-9\-\_\.]+(:?[A-Za-z0-9\-\_\.]+)$")
        if not (resource == "lambda" or resource == "any") or result:
            return result

    if resource == "lambda-event-mapping" or resource == "lambda" or resource == "any":
        result = input.match(r"^arn:(aws|aws-cn|aws-us-gov):lambda:\S+:\d{12}:event-source-mapping\/[A-Za-z0-9]+$")
        if not (resource == "lambda" or resource == "any") or result:
            return result

    if resource == "lambda-layer" or resource == "lambda" or resource == "any":
        result = input.match(r"^arn:(aws|aws-cn|aws-us-gov):lambda:\S+:\d{12}:layer:([A-Za-z0-9\-\_\.]+(:\d+)?)$")
        if not (resource == "lambda" or resource == "any") or result:
            return result

    if resource == "sts" or resource == "any":
        result = input.match(r"^arn:(aws|aws-cn|aws-us-gov):sts::\d{12}:[A-Za-z0-9\-\_\.]+\/([A-Za-z0-9\-\_\.]+(\/[A-Za-z0-9\-\_\.]+)?)$")
        if resource != "any" or result:
            return result

    if resource == "s3" or resource == "any":
        result = input.match(r"^arn:(aws|aws-cn|aws-us-gov):s3:::[a-zA-Z0-9_\-]+((\/.[^\/]+){1})?(?:\/\*)?$")
        if resource != "any" or result:
            return result

    if resource == "sqs" or resource == "any":
        result = input.match(r"^arn:(aws|aws-cn|aws-us-gov):sqs:\S+:\d{12}:[a-zA-Z0-9\-\_\.]+$")
        if resource != "any" or result:
            return result

    return False
	





from typing import List


def join(array: List[str], join_with: str = '') -> str:
    string = ""
    for i, item in enumerate(array):
        string = string + item
        if i != (len(array) - 1):
            string = string + join_with

    return string



from .utils.Classes.RegEx import RegEx
from .utils.assert_string import assert_string


def is_data_uri(input: str) -> bool:
    assert_string(input)

    valid_media_type = RegEx(r"^[a-z]+/[a-z0-9\-\+]+$", 'i')
    valid_attribute = RegEx(r"^[a-z\-]+=[a-z0-9\-]+$", 'i')
    valid_data = RegEx(r"^[a-z0-9!\$&'\(\)\*\+,;=\-\._~:@/\?%\s]*$", 'i')

    data = input.split(',')

    if len(data) < 2:
        return False

    attributes = data.pop(0).strip().split(';')
    scheme_and_media_type = attributes.pop(0)

    has_required_prefix = scheme_and_media_type[0:5] == 'data:'

    if not has_required_prefix:
        return False

    media_type = scheme_and_media_type[5:]

    is_valid_media_type = bool(valid_media_type.match(media_type))

    if media_type != '' and not is_valid_media_type:
        return False

    for i, attribute in enumerate(attributes):
        is_last_item = i == len(attributes) - 1
        is_base64 = attribute.lower() == 'base64'
        is_valid_attribute = valid_attribute.match(attribute)

        if not (is_last_item and is_base64) and not is_valid_attribute:
            return False

    for item in data:
        is_valid_data = valid_data.match(item)
        if not is_valid_data:
            return False

    return True



from .utils.Classes.RegEx import RegEx
from .utils.assert_string import assert_string
from .utils.index_of import index_of
from .utils.merge import merge

default_base64_options = {
    "url_safe": False
}


def is_base64(input: str, options={}) -> bool:
    assert_string(input)

    not_base64 = RegEx(r"[^A-Z0-9+/=]", "i")
    url_safe_base64 = RegEx(r"^[A-Z0-9_\-]*$", "i")

    options = merge(options, default_base64_options)
    input_length = len(input)

    if options["url_safe"]:
        return url_safe_base64.match(input)

    if (input_length % 4) != 0 or not_base64.match(input):
        return False

    first_padding_char = index_of(input, "=")
    return (
        first_padding_char == -1 or
        first_padding_char == input_length - 1 or
        (
            first_padding_char == input_length - 2 and
            input[input_length - 1] == "="
        )
    )



from typing import Union


def gather_then_check(input: str, min: Union[str, int, float, None]) -> bool:
    try:
        if min is None:
            return True

        input_num = float(input)
        min_num = float(min)

        return input_num >= min_num
    except:
        return False


def less_then_check(input: str, max: Union[str, int, float, None]) -> bool:
    try:
        if max is None:
            return True

        input_num = float(input)
        max_num = float(max)

        return input_num <= max_num
    except:
        return False



alpha = {
    'en-US': '^[A-Z]+$',
    'az-AZ': '^[A-VXYZÇƏĞİıÖŞÜ]+$',
    'bg-BG': '^[А-Я]+$',
    'cs-CZ': '^[A-ZÁČĎÉĚÍŇÓŘŠŤÚŮÝŽ]+$',
    'da-DK': '^[A-ZÆØÅ]+$',
    'de-DE': '^[A-ZÄÖÜß]+$',
    'el-GR': '^[Α-ώ]+$',
    'es-ES': '^[A-ZÁÉÍÑÓÚÜ]+$',
    'fa-IR': '^[ابپتثجچحخدذرزژسشصضطظعغفقکگلمنوهی]+$',
    'fi-FI': '^[A-ZÅÄÖ]+$',
    'fr-FR': '^[A-ZÀÂÆÇÉÈÊËÏÎÔŒÙÛÜŸ]+$',
    'it-IT': '^[A-ZÀÉÈÌÎÓÒÙ]+$',
    'nb-NO': '^[A-ZÆØÅ]+$',
    'nl-NL': '^[A-ZÁÉËÏÓÖÜÚ]+$',
    'nn-NO': '^[A-ZÆØÅ]+$',
    'hu-HU': '^[A-ZÁÉÍÓÖŐÚÜŰ]+$',
    'pl-PL': '^[A-ZĄĆĘŚŁŃÓŻŹ]+$',
    'pt-PT': '^[A-ZÃÁÀÂÄÇÉÊËÍÏÕÓÔÖÚÜ]+$',
    'ru-RU': '^[А-ЯЁ]+$',
    'sl-SI': '^[A-ZČĆĐŠŽ]+$',
    'sk-SK': '^[A-ZÁČĎÉÍŇÓŠŤÚÝŽĹŔĽÄÔ]+$',
    'sr-RS@latin': '^[A-ZČĆŽŠĐ]+$',
    'sr-RS': '^[А-ЯЂЈЉЊЋЏ]+$',
    'sv-SE': '^[A-ZÅÄÖ]+$',
    'th-TH': r'^[ก-๐\s]+$',
    'tr-TR': '^[A-ZÇĞİıÖŞÜ]+$',
    'uk-UA': '^[А-ЩЬЮЯЄIЇҐі]+$',
    'vi-VN': '^[A-ZÀÁẠẢÃÂẦẤẬẨẪĂẰẮẶẲẴĐÈÉẸẺẼÊỀẾỆỂỄÌÍỊỈĨÒÓỌỎÕÔỒỐỘỔỖƠỜỚỢỞỠÙÚỤỦŨƯỪỨỰỬỮỲÝỴỶỸ]+$',
    'ku-IQ': '^[ئابپتجچحخدرڕزژسشعغفڤقکگلڵمنوۆھەیێيطؤثآإأكضصةظذ]+$',
    'ar': '^[ءآأؤإئابةتثجحخدذرزسشصضطظعغفقكلمنهوىيًٌٍَُِّْٰ]+',
    'he': '^[א-ת]+',
    'fa': r'^[\'آاءأؤئبپتثجچحخدذرزژسشصضطظعغفقکگلمنوهةی\']+$',
    'hi-IN': r'^[\u0900-\u0961]+[\u0972-\u097F]*$',
}

alphanumeric = {
    'en-US': '^[0-9A-Z]+$',
    'az-AZ': '^[0-9A-VXYZÇƏĞİıÖŞÜ]+$',
    'bg-BG': '^[0-9А-Я]+$',
    'cs-CZ': '^[0-9A-ZÁČĎÉĚÍŇÓŘŠŤÚŮÝŽ]+$',
    'da-DK': '^[0-9A-ZÆØÅ]+$',
    'de-DE': '^[0-9A-ZÄÖÜß]+$',
    'el-GR': '^[0-9Α-ω]+$',
    'es-ES': '^[0-9A-ZÁÉÍÑÓÚÜ]+$',
    'fr-FR': '^[0-9A-ZÀÂÆÇÉÈÊËÏÎÔŒÙÛÜŸ]+$',
    'fi-FI': '^[0-9A-ZÅÄÖ]+$',
    'it-IT': '^[0-9A-ZÀÉÈÌÎÓÒÙ]+$',
    'hu-HU': '^[0-9A-ZÁÉÍÓÖŐÚÜŰ]+$',
    'nb-NO': '^[0-9A-ZÆØÅ]+$',
    'nl-NL': '^[0-9A-ZÁÉËÏÓÖÜÚ]+$',
    'nn-NO': '^[0-9A-ZÆØÅ]+$',
    'pl-PL': '^[0-9A-ZĄĆĘŚŁŃÓŻŹ]+$',
    'pt-PT': '^[0-9A-ZÃÁÀÂÄÇÉÊËÍÏÕÓÔÖÚÜ]+$',
    'ru-RU': '^[0-9А-ЯЁ]+$',
    'sl-SI': '^[0-9A-ZČĆĐŠŽ]+$',
    'sk-SK': '^[0-9A-ZÁČĎÉÍŇÓŠŤÚÝŽĹŔĽÄÔ]+$',
    'sr-RS@latin': '^[0-9A-ZČĆŽŠĐ]+$',
    'sr-RS': '^[0-9А-ЯЂЈЉЊЋЏ]+$',
    'sv-SE': '^[0-9A-ZÅÄÖ]+$',
    'th-TH': r'^[ก-๙\s]+$',
    'tr-TR': '^[0-9A-ZÇĞİıÖŞÜ]+$',
    'uk-UA': '^[0-9А-ЩЬЮЯЄIЇҐі]+$',
    'ku-IQ': '^[٠١٢٣٤٥٦٧٨٩0-9ئابپتجچحخدرڕزژسشعغفڤقکگلڵمنوۆھەیێيطؤثآإأكضصةظذ]+$',
    'vi-VN': '^[0-9A-ZÀÁẠẢÃÂẦẤẬẨẪĂẰẮẶẲẴĐÈÉẸẺẼÊỀẾỆỂỄÌÍỊỈĨÒÓỌỎÕÔỒỐỘỔỖƠỜỚỢỞỠÙÚỤỦŨƯỪỨỰỬỮỲÝỴỶỸ]+$',
    'ar': '^[٠١٢٣٤٥٦٧٨٩0-9ءآأؤإئابةتثجحخدذرزسشصضطظعغفقكلمنهوىيًٌٍَُِّْٰ]+',
    'he': '^[0-9א-ת]+',
    'fa': r'^[\'0-9آاءأؤئبپتثجچحخدذرزژسشصضطظعغفقکگلمنوهةی۱۲۳۴۵۶۷۸۹۰\']+$',
    'hi-IN': r'^[\u0900-\u0963]+[\u0966-\u097F]*$',
}

decimal = {
    'en-US': '.',
    'ar': '٫',
}

english_locales = ['AU', 'GB', 'HK', 'IN', 'NZ', 'ZA', 'ZM']

for i, locale in enumerate(english_locales):
    locale = "en-{}".format(english_locales[i])
    alpha[locale] = alpha['en-US']
    alphanumeric[locale] = alphanumeric['en-US']
    decimal[locale] = decimal['en-US']

arabic_locales = [
    'AE', 'BH', 'DZ', 'EG', 'IQ', 'JO', 'KW', 'LB', 'LY',
    'MA', 'QM', 'QA', 'SA', 'SD', 'SY', 'TN', 'YE',
]

for j, locale in enumerate(arabic_locales):
    locale = "ar-{}".format(arabic_locales[j])
    alpha[locale] = alpha['en-US']
    alphanumeric[locale] = alphanumeric['en-US']
    decimal[locale] = decimal['en-US']

farsi_locales = [
    'IR', 'AF',
]

for k, locale in enumerate(farsi_locales):
    locale = "fa-{}".format(farsi_locales[k])
    alpha[locale] = alpha['en-US']
    alphanumeric[locale] = alphanumeric['en-US']
    decimal[locale] = decimal['en-US']

dot_decimal = ['ar-EG', 'ar-LB', 'ar-LY']

for n, _ in enumerate(farsi_locales):
    decimal[dot_decimal[n]] = decimal['en-US']

comma_decimal = [
    'bg-BG', 'cs-CZ', 'da-DK', 'de-DE', 'el-GR', 'en-ZM', 'es-ES', 'fr-CA', 'fr-FR', 'fi-FI',
    'id-ID', 'it-IT', 'ku-IQ', 'hi-IN', 'hu-HU', 'nb-NO', 'nn-NO', 'nl-NL', 'pl-PL', 'pt-PT',
    'ru-RU', 'sl-SI', 'sr-RS@latin', 'sr-RS', 'sv-SE', 'tr-TR', 'uk-UA', 'vi-VN',
]

for m, _ in enumerate(comma_decimal):
    decimal[comma_decimal[m]] = decimal['en-US']

alpha['fr-CA'] = alpha['fr-FR']
alphanumeric['fr-CA'] = alphanumeric['fr-FR']

alpha['pt-BR'] = alpha['pt-PT']
alphanumeric['pt-BR'] = alphanumeric['pt-PT']
decimal['pt-BR'] = decimal['pt-PT']

alpha['pl-Pl'] = alpha['pl-PL']
alphanumeric['pl-Pl'] = alphanumeric['pl-PL']
decimal['pl-Pl'] = decimal['pl-PL']

alpha['fa-AF'] = alpha['fa']



import unittest

from pyvalidator.is_alpha import is_alpha
from . import print_test_ok


class TestIsAlpha(unittest.TestCase):
    def valid_check(self, items, locale=None, options={}):
        for item in items:
            try:
                self.assertTrue(is_alpha(item, locale, options))
            except Exception as e:
                print(f'failed for input: {item}')
                raise e

    def invalid_check(self, items, locale=None, options={}):
        for item in items:
            try:
                self.assertFalse(is_alpha(item, locale, options))
            except Exception as e:
                print(f'failed for input: {item}')
                raise e

    def test_valid_alpha_strings(self):
        valid = [
            'abc',
            'ABC',
            'FoObar',
        ]

        self.valid_check(valid)
        print_test_ok()

    def test_invalid_alpha_strings(self):
        invalid = [
            'abc1',
            '  foo  ',
            'FÜübar',
            'Jön',
        ]

        self.invalid_check(invalid)
        print_test_ok()

    def test_valid_alpha_strings_with_ignored_characters(self):
        valid = [
            'en-US',
            'this is a valid alpha string',
            'us/usa',
        ]
        self.valid_check(valid, 'en-US', {"ignore": '- /'})
        print_test_ok()

    def test_invalid_alpha_strings_with_ignored_characters(self):
        invalid = [
            '1. this is not a valid alpha string',
            'this$is also not a valid.alpha string',
            'this is also not a valid alpha string.',
        ]
        self.invalid_check(invalid, 'en-US', {"ignore": '- /'})
        print_test_ok()

    def test_should_throw_for_invalid_ignore_matcher(self):
        self.assertRaises(Exception, is_alpha, ['this is also not a valid alpha string.', 'en-US', {"ignore": 123}])
        print_test_ok()

    def test_valid_azerbaijani_string(self):
        valid = [
            'Azərbaycan',
            'Bakı',
            'üöğıəçş',
            'sizAzərbaycanlaşdırılmışlardansınızmı',
            'dahaBirDüzgünString',
            'abcçdeəfgğhxıijkqlmnoöprsştuüvyz',
        ]

        self.valid_check(valid, 'az-AZ')
        print_test_ok()

    def test_invalid_azerbaijani_string(self):
        invalid = [
            'rəqəm1',
            '  foo  ',
            '',
            'ab(cd)',
            'simvol@',
            'wəkil',
        ]

        self.invalid_check(invalid, 'az-AZ')
        print_test_ok()

    def test_valid_bulgarian_string(self):
        valid = [
            'абв',
            'АБВ',
            'жаба',
            'яГоДа',
        ]

        self.valid_check(valid, 'bg-BG')
        print_test_ok()

    def test_invalid_bulgarian_string(self):
        invalid = [
            'abc1',
            '  foo  ',
            '',
            'ЁЧПС',
            '_аз_обичам_обувки_',
            'ехо!',
        ]

        self.invalid_check(invalid, 'bg-BG')
        print_test_ok()

    def test_valid_czech_string(self):
        valid = [
            'žluťoučký',
            'KŮŇ',
            'Pěl',
            'Ďábelské',
            'ódy',
        ]

        self.valid_check(valid, 'cs-CZ')
        print_test_ok()

    def test_invalid_czech_string(self):
        invalid = [
            'ábc1',
            '  fůj  ',
            '',
        ]

        self.invalid_check(invalid, 'cs-CZ')
        print_test_ok()

    def test_valid_slovak_string(self):
        valid = [
            'môj',
            'ľúbím',
            'mäkčeň',
            'stĹp',
            'vŕba',
            'ňorimberk',
            'ťava',
            'žanéta',
            'Ďábelské',
            'ódy',
        ]

        self.valid_check(valid, 'sk-SK')
        print_test_ok()

    def test_invalid_slovak_string(self):
        invalid = [
            '1moj',
            '你好世界',
            '  Привет мир  ',
            'مرحبا العا '
        ]

        self.invalid_check(invalid, 'sk-SK')
        print_test_ok()

    def test_valid_danish_string(self):
        valid = [
            'aøå',
            'Ære',
            'Øre',
            'Åre',
        ]

        self.valid_check(valid, 'da-DK')
        print_test_ok()

    def test_invalid_danish_string(self):
        invalid = [
            'äbc123',
            'ÄBC11',
            '',
        ]

        self.invalid_check(invalid, 'da-DK')
        print_test_ok()

    def test_valid_dutch_string(self):
        valid = [
            'Kán',
            'één',
            'vóór',
            'nú',
            'héél',
        ]

        self.valid_check(valid, 'nl-NL')
        print_test_ok()

    def test_invalid_dutch_string(self):
        invalid = [
            'äca ',
            'abcß',
            'Øre',
        ]

        self.invalid_check(invalid, 'nl-NL')
        print_test_ok()

    def test_valid_german_string(self):
        valid = [
            'äbc',
            'ÄBC',
            'FöÖbär',
            'Heiß',
        ]

        self.valid_check(valid, 'de-DE')
        print_test_ok()

    def test_invalid_german_string(self):
        invalid = [
            'äbc1',
            '  föö  ',
            '',
        ]

        self.invalid_check(invalid, 'de-DE')
        print_test_ok()

    def test_valid_hungarian_string(self):
        valid = [
            'árvíztűrőtükörfúrógép',
            'ÁRVÍZTŰRŐTÜKÖRFÚRÓGÉP',
        ]

        self.valid_check(valid, 'hu-HU')
        print_test_ok()

    def test_invalid_hungarian_string(self):
        invalid = [
            'äbc1',
            '  fäö  ',
            'Heiß',
            '',
        ]

        self.invalid_check(invalid, 'hu-HU')
        print_test_ok()

    def test_valid_portuguese_string(self):
        valid = [
            'palíndromo',
            'órgão',
            'qwértyúão',
            'àäãcëüïÄÏÜ',
        ]

        self.valid_check(valid, 'pt-PT')
        print_test_ok()

    def test_invalid_portuguese_string(self):
        invalid = [
            '12abc',
            'Heiß',
            'Øre',
            'æøå',
            '',
        ]

        self.invalid_check(invalid, 'pt-PT')
        print_test_ok()

    def test_valid_italian_string(self):
        valid = [
            'àéèìîóòù',
            'correnti',
            'DEFINIZIONE',
            'compilazione',
            'metró',
            'pèsca',
            'PÉSCA',
            'genî',
        ]

        self.valid_check(valid, 'it-IT')
        print_test_ok()

    def test_invalid_italian_string(self):
        invalid = [
            'äbc123',
            'ÄBC11',
            'æøå',
            '',
        ]

        self.invalid_check(invalid, 'it-IT')
        print_test_ok()

    def test_valid_vietnamese_string(self):
        valid = [
            'thiến',
            'nghiêng',
            'xin',
            'chào',
            'thế',
            'giới',
        ]

        self.valid_check(valid, 'vi-VN')
        print_test_ok()

    def test_invalid_vietnamese_string(self):
        invalid = [
            'thầy3',
            'Ba gà',
            '',
        ]

        self.invalid_check(invalid, 'vi-VN')
        print_test_ok()

    def test_valid_arabic_string(self):
        valid = [
            'أبت',
            'اَبِتَثّجً',
        ]

        self.valid_check(valid, 'ar')
        print_test_ok()

    def test_invalid_arabic_string(self):
        invalid = [
            '١٢٣أبت',
            '١٢٣',
            'abc1',
            '  foo  ',
            '',
            'ÄBC',
            'FÜübar',
            'Jön',
            'Heiß',
        ]

        self.invalid_check(invalid, 'ar')
        print_test_ok()

    def test_valid_finnish_string(self):
        valid = [
            'äiti',
            'Öljy',
            'Åke',
            'testÖ',
        ]

        self.valid_check(valid, 'fi-FI')
        print_test_ok()

    def test_invalid_finnish_string(self):
        invalid = [
            'AİıÖöÇçŞşĞğÜüZ',
            'äöå123',
            '',
        ]

        self.invalid_check(invalid, 'fi-FI')
        print_test_ok()

    def test_valid_polish_string(self):
        valid = [
            'kreską',
            'zamknięte',
            'zwykłe',
            'kropką',
            'przyjęły',
            'święty',
            'Pozwól',
        ]

        self.valid_check(valid, 'pl-PL')
        print_test_ok()

    def test_invalid_polish_string(self):
        invalid = [
            '12řiď ',
            'blé!!',
            'föö!2!',
        ]

        self.invalid_check(invalid, 'pl-PL')
        print_test_ok()



import unittest

from pyvalidator.is_date import is_date
from . import print_test_ok


class TestIsDate(unittest.TestCase):

    def test_valid_date(self):
        for i in [
            '2020/02/29',
            '2004-02-29',
            '2011-12-13',
            '1911-12-13',
        ]:
            self.assertTrue(is_date(i))
        print_test_ok()

    def test_invalid_date(self):
        for i in [
            '15072002',
            '2020-02-30',
            '2011-12/13',
            '2011/2/13',
            '',
            'asd',
        ]:
            self.assertFalse(is_date(i))
        print_test_ok()

    def test_valid_date_custom_format(self):
        for i in [
            ['12/20/19', {"format": "MM/DD/YY"}],
            ['12/20/19', {"format": "mm/dd/yy"}],
            ['02-04-04', {"format": "MM/DD/YY"}],
            ['02-04-04', {"format": "mm-DD-yy"}],
        ]:
            self.assertTrue(is_date(*i))
        print_test_ok()

    def test_invalid_date_custom_format(self):
        for i in [
            ['2/20/29', {"format": "MM/DD/YY"}],
            ['02/20/29', {"format": "MM/DD/YYYY"}],
        ]:
            self.assertFalse(is_date(*i))
        print_test_ok()

    def test_valid_date_strict_mode(self):
        for i in [
            '2020/02/29',
            '1920/02/29',
        ]:
            self.assertTrue(is_date(i, {"strict_mode": True}))
        print_test_ok()

    def test_invalid_date_strict_mode(self):
        for i in [
            '15072002',
            '2020-02-30',
            '2004-02-29',
            '2011-12/13',
        ]:
            self.assertFalse(is_date(i, {"strict_mode": True}))
        print_test_ok()

    def test_valid_date_strict_mode_hyphen_format(self):
        for i in [
            '2020-02-29',
        ]:
            self.assertTrue(is_date(i, {"strict_mode": True, "format": 'YYYY-MM-DD'}))
        print_test_ok()

    def test_invalid_date_strict_mode_hyphen_format(self):
        for i in [
            '2020/02/29',
            '2019/09/30',
        ]:
            self.assertFalse(is_date(i, {"strict_mode": True, "format": 'YYYY-MM-DD'}))
        print_test_ok()

    def test_valid_date_custom_delimiter(self):
        for i in [
            ['2020.02.29', {"delimiters": ['.']}],
            ['2020,02,29', {"delimiters": [',']}],
            ['02,11,2019', {"delimiters": [','], "format": 'DD/MM/YYYY'}],
            ['02,11,19', {"delimiters": [','], "format": 'DD/MM/YY'}],
            ['02|11|19', {"delimiters": ['|'], "format": 'YY|MM|DD'}],
            ['02;11;19', {"delimiters": [';'], "format": 'YY;MM;DD'}],
            ['02:11:19', {"delimiters": [':'], "format": 'YY:MM:DD'}],
        ]:
            self.assertTrue(is_date(*i))
        print_test_ok()

    def test_invalid_date_custom_delimiter(self):
        for i in [
            ['2020,02,29', {"delimiters": ['.']}],
            ['2020.02.29', {"delimiters": [',']}],
            ['02,11,2019', {"delimiters": [','], "format": 'DD/MM/YY'}],
        ]:
            self.assertFalse(is_date(*i))
        print_test_ok()

    def test_valid_date_custom_delimiter_strict_mode(self):
        self.assertTrue(is_date('2020.02.29', {"delimiters": ['.'], "strict_mode": True, "format": 'YYYY.MM.DD'}))
        self.assertTrue(is_date('02.29.20', {"delimiters": ['.'], "strict_mode": True, "format": 'MM.DD.YY'}))
        print_test_ok()

    def test_invalid_date_custom_delimiter_strict_mode(self):
        self.assertFalse(is_date('2020.02.29', {"delimiters": ['.'], "strict_mode": True, "format": 'YYYY,MM,DD'}))
        print_test_ok()

    def test_rais_error_for_unsuported_date_format(self):
        for i in [
            ['2020/02/29', {"format": 'MM/YY/DD'}],
            ['2020/02/29', {"format": ''}],
        ]:
            self.assertRaises(Exception, is_date, i)
        print_test_ok()



from .alpha import decimal
from .utils.Classes.RegEx import RegEx
from .utils.assert_string import assert_string
from .utils.includes import includes
from .utils.math import gather_then_check, less_then_check
from .utils.merge import merge

__default_options = {
    "min": None,
    "max": None,
    "lt": None,
    "gt": None,
    "locale": None,
}


def is_float(input: str, options=__default_options) -> bool:
    input = assert_string(input)

    options = merge(options, __default_options)

    if includes(['+', '-', '.', ''], input):
        return False

    float_pattern = RegEx(r"^(?:[-+])?(?:[0-9]+)?(?:\{}[0-9]*)?(?:[eE][\+\-]?(?:[0-9]+))?$".format(decimal[options["locale"]] if bool(options["locale"]) else '.'))

    transformed_input = input.sub(',', '.').sub('٫', '.')

    is_valid = float_pattern.match(input)

    min_valid = gather_then_check(transformed_input, options["min"])
    max_valid = less_then_check(transformed_input, options["max"])
    gt_valid = gather_then_check(transformed_input, options["gt"])
    lt_valid = less_then_check(transformed_input, options["lt"])

    return is_valid and min_valid and max_valid and lt_valid and gt_valid



import datetime
import unittest

from pyvalidator.is_after import is_after
from . import print_test_ok


class TestIsAfter(unittest.TestCase):

    def test_valid_after_dates_against_a_start_date(self):
        for i in [
            ['2011-08-04', '2011-08-03'],
            [datetime.datetime.now().isoformat(), '2011-08-03'],
        ]:
            self.assertTrue(is_after(*i))
        print_test_ok()

    def test_invalid_after_dates_against_a_start_date(self):
        for i in [
            ['2011-07-02', '2011-08-03'],
            ['2011-08-03', '2011-08-03'],
            [datetime.datetime.utcfromtimestamp(0).isoformat(), '2011-08-03'],
            ['foo', '2011-08-03'],
        ]:
            self.assertFalse(is_after(*i))
        print_test_ok()



from .Classes.String import String


def assert_string(input: str) -> String:
    is_string = type(input).__name__ == 'str' or isinstance(input, str)

    if not is_string:
        invalid_type = type(input).__name__
        raise ValueError(f"Expected a string but received a {invalid_type}")

    return String(input)



from typing import TypedDict
from .utils.assert_string import assert_string
from .utils.merge import merge


class IsPalindromeOptions(TypedDict):
	insensitive: bool

default_is_palindrome_options: IsPalindromeOptions = {
  "insensitive": True,
}

def is_palindrome(input: str, options: IsPalindromeOptions = {}) -> bool:
    input = assert_string(input)

    options = merge(options, default_is_palindrome_options)

    input = input.sub(r'[^a-zA-Z\d\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]', '')

    if not options['insensitive']: 
        return input == input[::-1]

    return input.lower() == input[::-1].lower()



from .utils.Classes.RegEx import RegEx
from .utils.assert_string import assert_string


def is_ascii(input: str) -> bool:
    pattern = RegEx(r"^[\x00-\x7F]+$")
    input = assert_string(input)
    return pattern.match(input)



from typing import List, TypedDict

from .utils.assert_string import assert_string
from .utils.merge import merge


class IsUrlOptions(TypedDict):
    no_scheme: bool
    with_no_path: bool
    insensitive: bool
    top_level_domains: List[str]
    domains: List[str]


default_email_options: IsUrlOptions = {
    "no_scheme": False,
    "with_no_path": False,
    "insensitive": True,
    "top_level_domains": [],
    "domains": [],
}


def is_url(input: str, options={}) -> bool:
    input = assert_string(input)

    options = merge(options, default_email_options)

    domains = options["domains"]
    top_level_domains = options["top_level_domains"]
    insensitive = options["insensitive"]

    domains_pattern = ""

    if len(domains) > 0:
        for domain in domains:
            if not domains_pattern:
                domains_pattern = domain.strip()
            else:
                domains_pattern = f"{domains_pattern}|{domain}"
        domains_pattern = rf"(?=.*({domains_pattern})\.)"

    top_level_domain_pattern = ""

    if len(top_level_domains) > 0:
        for domain in top_level_domains:
            if not top_level_domain_pattern:
                top_level_domain_pattern = domain.strip()
            else:
                top_level_domain_pattern = f"{top_level_domain_pattern}|{domain}"
        top_level_domain_pattern = rf"(\.({top_level_domain_pattern}))"
    else:
        top_level_domain_pattern = r"\.[a-z]{2,63}"

    protocol_pattern = "" if options["no_scheme"] else r"((http(s)?)://)?"

    limit_domain_length = "{1,255}"
    base_url_pattern = fr"{protocol_pattern}(www\.)?(?!-)[(\-a-z0-9.]{limit_domain_length}[a-z0-9]{top_level_domain_pattern}"

    path_pattern = r"/?" if options["with_no_path"] else r"\b(/[-a-zA-Z0-9()@:%_\+.~#?&//=]*)?"

    pattern = fr"^({domains_pattern}{base_url_pattern}{path_pattern})$"

    return input.match(pattern, 'i') if insensitive else input.match(pattern)



from .alpha import decimal
from .utils.Classes.RegEx import RegEx
from .utils.assert_string import assert_string
from .utils.includes import includesNot
from .utils.merge import merge


def decimal_regexp(options):
    local = options["locale"]
    local_decimal = decimal[local]
    decimal_digits = options["decimal_digits"]
    require_decimal = '' if options["force_decimal"] else '?'
    regex = r"^[-+]?([0-9]+)?(\{}[0-9]{{".format(local_decimal) + "{}}}){}$".format(decimal_digits, require_decimal)
    return RegEx(regex)


default_decimal_options = {
    "force_decimal": False,
    "decimal_digits": '1,',
    "locale": 'en-US',
}

blacklist = ['', '-', '+']


def is_decimal(input: str, options={}) -> bool:
    input = assert_string(input)

    input = input.rstrip()

    options = merge(options, default_decimal_options)

    if options["locale"] not in decimal:
        raise ValueError("Invalid locale '{}'".format(options["locale"]))

    without_white_space = input.sub(" ", '')
    return includesNot(blacklist, without_white_space) and bool(decimal_regexp(options).match(input))



from .utils.assert_string import assert_string


def is_css_unit(input: str) -> bool:
    input = assert_string(input)

    pattern = r"^((-?(\d*\.)?\d+)((px)|(em)|(%)|(ex)|(ch)|(rem)|(vw)|(vh)|(vmin)|(vmax)|(cm)|(mm)|(in)|(pt)|(pc))|0)$"

    return input.match(pattern)



import json
from typing import TypedDict

from .utils.Classes.List import List
from .utils.assert_string import assert_string
from .utils.merge import merge


class IsJsonOptions(TypedDict):
    allow_primitives: bool


default_json_options: IsJsonOptions = {
    "allow_primitives": False,
}


def is_json(input: str, options: IsJsonOptions = {}) -> bool:
    input = assert_string(input)
    options = merge(options, default_json_options)
    try:
        parsed = json.loads(input)
        invalid_json_types = List(['int', 'float'])
        if invalid_json_types.includes(type(parsed).__name__):
            return False
        primitives = List(['null', 'false', 'true'])
        if not options["allow_primitives"] and primitives.includes(input):
            return False
        return True
    except:
        return False



from .utils.Classes.RegEx import RegEx
from .utils.assert_string import assert_string

passport_regex_by_country_code = {
    "AM": r"^[A-Z]{2}\d{7}$",  # ARMENIA
    "AR": r"^[A-Z]{3}\d{6}$",  # ARGENTINA
    "AT": r"^[A-Z]\d{7}$",  # AUSTRIA
    "AU": r"^[A-Z]\d{7}$",  # AUSTRALIA
    "BA": r"^[A-Z]\d{7}$",  # BOSNIA AND HERZEGOVINA,
    "BE": r"^[A-Z]{2}\d{6}$",  # BELGIUM
    "BG": r"^\d{9}$",  # BULGARIA
    "BR": r"^[A-Z]{2}\d{6}$",  # BRAZIL
    "BY": r"^[A-Z]{2}\d{7}$",  # BELARUS
    "CA": r"^[A-Z]{2}\d{6}$",  # CANADA
    "CH": r"^[A-Z]\d{7}$",  # SWITZERLAND
    "CN": r"^G\d{8}$|^E(?![IO])[A-Z0-9]\d{7}$",  # CHINA
    "CY": r"^[A-Z](\d{6}|\d{8})$",  # CYPRUS
    "CZ": r"^\d{8}$",  # CZECH REPUBLIC
    "DE": "^[CFGHJKLMNPRTVWXYZ0-9]{9}$",  # GERMANY
    "DK": r"^\d{9}$",  # DENMARK
    "DZ": r"^\d{9}$",  # ALGERIA
    "EE": r"^([A-Z]\d{7}|[A-Z]{2}\d{7})$",  # ESTONIA
    "ES": r"^[A-Z0-9]{2}([A-Z0-9]?)\d{6}$",  # SPAIN
    "FI": r"^[A-Z]{2}\d{7}$",  # FINLAND
    "FR": r"^\d{2}[A-Z]{2}\d{5}$",  # FRANCE
    "GB": r"^\d{9}$",  # UNITED KINGDOM
    "GR": r"^[A-Z]{2}\d{7}$",  # GREECE
    "HR": r"^\d{9}$",  # CROATIA
    "HU": r"^[A-Z]{2}(\d{6}|\d{7})$",  # HUNGARY
    "IE": r"^[A-Z0-9]{2}\d{7}$",  # IRELAND
    "IN": r"^[A-Z]{1}-?\d{7}$",  # INDIA
    "ID": r"^[A-C]\d{7}$",  # INDONESIA
    "IR": r"^[A-Z]\d{8}$",  # IRAN
    "IS": r"^(A)\d{7}$",  # ICELAND
    "IT": r"^[A-Z0-9]{2}\d{7}$",  # ITALY
    "JP": r"^[A-Z]{2}\d{7}$",  # JAPAN
    "KR": r"^[MS]\d{8}$",  # SOUTH KOREA, REPUBLIC OF KOREA
    "LT": "^[A-Z0-9]{8}$",  # LITHUANIA
    "LU": "^[A-Z0-9]{8}$",  # LUXEMBURG
    "LV": r"^[A-Z0-9]{2}\d{7}$",  # LATVIA
    "LY": "^[A-Z0-9]{8}$",  # LIBYA
    "MT": r"^\d{7}$",  # MALTA
    "MZ": r"^([A-Z]{2}\d{7})|(\d{2}[A-Z]{2}\d{5})$",  # MOZAMBIQUE
    "MY": r"^[AHK]\d{8}$",  # MALAYSIA
    "NL": r"^[A-Z]{2}[A-Z0-9]{6}\d$",  # NETHERLANDS
    "PL": r"^[A-Z]{2}\d{7}$",  # POLAND
    "PT": r"^[A-Z]\d{6}$",  # PORTUGAL
    "RO": r"^\d{8,9}$",  # ROMANIA
    "RS": r"^\d{9}$",  # SERBIA,
    "RU": r"^\d{9}$",  # RUSSIAN FEDERATION
    "SE": r"^\d{8}$",  # SWEDEN
    "SL": r"^(P)[A-Z]\d{7}$",  # SLOVENIA
    "SK": r"^[0-9A-Z]\d{7}$",  # SLOVAKIA
    "TR": r"^[A-Z]\d{8}$",  # TURKEY
    "UA": r"^[A-Z]{2}\d{6}$",  # UKRAINE
    "US": r"^\d{9}$",  # UNITED STATES
}


def is_passport_number(input: str, country_code: str) -> bool:
    input = assert_string(input).sub(RegEx(r"\s", "g"), "").upper()

    is_valid_country = country_code in passport_regex_by_country_code

    if not is_valid_country:
        return False

    return input.match(passport_regex_by_country_code[country_code])



import unittest

from pyvalidator.is_palindrome import is_palindrome
from . import print_test_ok


class TestIsPalindrome(unittest.TestCase):
    def valid_check(self, items, options={}):
        for item in items:
            try:
                self.assertTrue(is_palindrome(item, options))
            except Exception as e:
                print(f'failed for input: {item}')
                raise e

    def invalid_check(self, items, options={}):
        for item in items:
            try:
                self.assertFalse(is_palindrome(item, options))
            except Exception as e:
                print(f'failed for input: {item}')
                raise e

    def test_valid_palindrome(self):
        valid_items = [
            '',
            'A',
            'EVE',
            'radar',
            'reviver',
            'top spot',
            'ROTATOR',
            'step not on pets.',
            'MADAM I\'M ADAM',
            'no lemon, NO Melon'
            'No LeMoN, nomelon',
            'Able was I, ere I saw Elba!',
            'Was it a car or a cat I saw',
            'Eine treue Familie bei Lima feuerte nie',
            'Die Liebe ist Sieger; stets rege ist sie bei Leid',
            'O Genie, der Herr ehre Dein Ego',
            'Elu par cette crapule',
            'Tu l\'as trop ecrase cesar ce port salut.',
            'A l\'autel elle alla, elle le tua la',
            'Caser vite ce palindrome ne mord ni lape cet ivre sac',
            'isälläsi',
            'Ana, kanna kana',
            'Hupaisa asia, Puh',
            'Isorikas sika sökösakissa kirosi',
            'I jogurt ujutru goji?',
            'УЈАК ИМА РАДАР  А МИ КАЈУ',
            'око',
            '12321'

        ]
        self.valid_check(valid_items)
        print_test_ok()

    def test_invalid_palindrome(self):
        invalid_items = [
            'asd',
            'hello, world'
            'this is not a palindrome',
            '12345',
            'ок0'

        ]
        self.invalid_check(invalid_items)
        print_test_ok()

    def test_invalid_case_sensitive_palindrome(self):
        invalid_items = [
            'No LeMoN, nomelon',
            'Was it a car or a cat I saw',
            'isäLläSi',
        ]
        self.invalid_check(invalid_items, {"insensitive": False})
        print_test_ok()



import unittest

from pyvalidator.is_prime import is_prime
from . import print_test_ok


class TestIsPrime(unittest.TestCase):
    def valid_check(self, items):
        for item in items:
            try:
                self.assertTrue(is_prime(item))
            except Exception as e:
                print(f'failed for input: {item}')
                raise e

    def invalid_check(self, items):
        for item in items:
            try:
                self.assertFalse(is_prime(item))
            except Exception as e:
                print(f'failed for input: {item}')
                raise e

    def test_valid_prime(self):
        valid_items = [
            '2',
            '3',
            '5',
            '7',
            '11',
            '13',
            '17',
            '19',
            '23',
            '29',
            '31',
            2,
            3,
            5,
            7,
            11,
            13,
            17,
            19,
            23,
            29,
            31,
            37,
            41,
            43,
            47,
            53,
            59,
            61,
            67,
            71,
            73,
            79,
            83,
            89,
            97,
            21577
        ]
        self.valid_check(valid_items)
        print_test_ok()

    def test_invalid_prime(self):
        invalid_items = [
            1,
            4,
            6,
            8,
            9,
            10,
            12,
            14,
            15,
            16,
            18,
            20,
            21,
            22,
            24,
            25,
            26,
            27,
            28,
            30,
            32,
            33,
            34,
            35,
            36,
            38,
            39,
            40,
            42,
            44,
            45,
            46,
            48,
            49,
            50,
            51,
            52,
            54,
            55,
            56,
            57,
            58,
            60,
            62,
            63,
            '51',
            '52',
            '54',
            '55',
            '56',
            '57',
            '58',
            '60',
            '62',
            '63',
        ]
        self.invalid_check(invalid_items)
        print_test_ok()



from typing import TypedDict

from .utils.Classes.RegEx import RegEx
from .utils.Classes.String import String
from .utils.assert_string import assert_string
from .utils.merge import merge


class IsFqdnOptions(TypedDict):
    require_tld: bool
    allow_underscores: bool
    allow_trailing_dot: bool
    allow_numeric_tld: bool
    allow_wildcard: bool


default_fqdn_options: IsFqdnOptions = {
    "require_tld": True,
    "allow_underscores": False,
    "allow_trailing_dot": False,
    "allow_numeric_tld": False,
    "allow_wildcard": False,
}


def is_fqdn(input: str, options: IsFqdnOptions = {}) -> bool:
    input = assert_string(input)
    options = merge(options, default_fqdn_options)

    if options["allow_trailing_dot"] and input[input.length - 1] == '.':
        input = input.substring(0, input.length - 1)

    if options["allow_wildcard"] and input.index('*.') == 0:
        input = input.substring(2)

    parts = input.split('.')
    tld = String(parts[parts.length - 1])

    if options["require_tld"]:
        if parts.length < 2:
            return False
        if not tld.match(RegEx(r"^([a-z\u00A1-\u00A8\u00AA-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]{2,}|xn[a-z0-9-]{2,})$", 'i')):
            return False

        if RegEx(r'\s').findall(tld):
            return False

    if not options["allow_numeric_tld"] and tld.match(r"^\d+$"):
        return False

    for part in parts:
        part = String(part)
        if part.length > 63:
            return False
        if not part.match(r"^[a-z_\u00a1-\uffff0-9-]+$", 'i'):
            return False
        if part.match(r"[\uff01-\uff5e]"):
            return False
        if part.match(r"^-|-$"):
            return False
        if not options["allow_underscores"] and part.match(r"_"):
            return False

    return True



import unittest

from pyvalidator.is_md5 import is_md5
from . import print_test_ok


class TestIsMd5(unittest.TestCase):

    def test_valid_md5(self):
        for i in [
            'd94f3f016ae679c3008de268209132f0',
            '751adbc511ccbe8edf23d486fa4581c0',
            '88dae00e614d8f24cfd5a8b3f8002e99',
            '0bf1c35032a71a14c2f719e5a14c1e99',
        ]:
            self.assertTrue(is_md5(i))
        print_test_ok()

    def test_invalid_md5(self):
        for i in [
            'KYT0bf1c35032a71a14c2f719e5a14c1',
            'q94375dj93458w31',
            '39485729341',
            '%&FHKJFvc',
            '',
        ]:
            self.assertFalse(is_md5(i))
        print_test_ok()

    def test_fail_md5(self):
        self.assertRaises(Exception, is_md5, None)
        self.assertRaises(Exception, is_md5, {})
        print_test_ok()



import unittest

from pyvalidator.is_magnet_uri import is_magnet_uri
from . import print_test_ok


class TestIsMagnetURI(unittest.TestCase):

    def test_valid_magnet_uri(self):
        for i in [
            'magnet:?xt.1=urn:sha1:ABCDEFGHIJKLMNOPQRSTUVWXYZ123456&xt.2=urn:sha1:ABCDEFGHIJKLMNOPQRSTUVWXYZ123456',
            'magnet:?xt=urn:btih:ABCDEFGHIJKLMNOPQRSTUVWXYZ12345678901234&dn=helloword2000&tr=udp://helloworld:1337/announce',
            'magnet:?xt=urn:btih:ABCDEFGHIJKLMNOPQRSTUVWXYZ12345678901234&dn=foo',
            'magnet:?xt=urn:btih:ABCDEFGHIJKLMNOPQRSTUVWXYZ12345678901234&dn=&tr=&nonexisting=hello world',
            'magnet:?xt=urn:md5:ABCDEFGHIJKLMNOPQRSTUVWXYZ123456',
            'magnet:?xt=urn:tree:tiger:ABCDEFGHIJKLMNOPQRSTUVWXYZ123456',
            'magnet:?xt=urn:ed2k:ABCDEFGHIJKLMNOPQRSTUVWXYZ12345678901234',
            'magnet:?tr=udp://helloworld:1337/announce&xt=urn:btih:ABCDEFGHIJKLMNOPQRSTUVWXYZ12345678901234',
            'magnet:?xt=urn:btmh:1220caf1e1c30e81cb361b9ee167c4aa64228a7fa4fa9f6105232b28ad099f3a302e',
        ]:
            self.assertTrue(is_magnet_uri(i))
        print_test_ok()

    def test_invalid_magnet_uri(self):
        for i in [
            ':?xt=urn:btih:ABCDEFGHIJKLMNOPQRSTUVWXYZ12345678901234',
            'xt=urn:btih:ABCDEFGHIJKLMNOPQRSTUVWXYZ12345678901234',
            'magneta:?xt=urn:btih:ABCDEFGHIJKLMNOPQRSTUVWXYZ12345678901234',
            'magnet:?xt=uarn:btih:ABCDEFGHIJKLMNOPQRSTUVWXYZ12345678901234',
            'magnet:?xt=urn:btihz',
            'magnet::?xt=urn:btih:UHWY2892JNEJ2GTEYOMDNU67E8ICGICYE92JDUGH',
            'magnet:?xt:btih:ABCDEFGHIJKLMNOPQRSTUVWXYZ',
            'magnet:?xt:urn:nonexisting:ABCDEFGHIJKLMNOPQRSTUVWXYZ12345678901234',
            'magnet:?xt.2=urn:btih:ABCDEFGHIJKLMNOPQRSTUVWXYZ12345678901234',
            'magnet:?xt=urn:ed2k:ABCDEFGHIJKLMNOPQRSTUVWXYZ12345678901234567890123456789ABCD',
            'magnet:?xt=urn:btmh:1120caf1e1c30e81cb361b9ee167c4aa64228a7fa4fa9f6105232b28ad099f3a302e',
            'magnet:?ttxt=urn:btmh:1220caf1e1c30e81cb361b9ee167c4aa64228a7fa4fa9f6105232b28ad099f3a302e',
        ]:
            self.assertFalse(is_magnet_uri(i))
        print_test_ok()



import unittest

from pyvalidator.is_lowercase import is_lowercase
from . import print_test_ok


class TestIsLowercase(unittest.TestCase):

    def test_valid_lowercase(self):
        for i in [
            'ccc',
            'qwerty',
            'asd123',
            'hello world',
            '   .',
        ]:
            self.assertTrue(is_lowercase(i))
        print_test_ok()

    def test_invalid_lowercase(self):
        for i in [
            'hello WoRLD',
            'HELLO WORLd',
            'ASD123',
        ]:
            self.assertFalse(is_lowercase(i))
        print_test_ok()



from typing import Union

__map = map


def map(item: Union[str, list], cb):
    cb_num_arg = cb.__code__.co_argcount
    if cb_num_arg == 1:
        return __map(cb, item)
    elif cb_num_arg == 2:
        return __map(cb, item, range(item.length))
    else:
        empty_args = ['None' * item.length] * (cb_num_arg - 2)
        arguments = [cb, item, range(item.length), *empty_args]
        return __map(*arguments)



from typing import Union

from .is_ip import is_ip
from .utils.Classes.String import String
from .utils.assert_string import assert_string

subnet_maybe = r"^\d{1,3}$"
subnet_v4 = 32
subnet_v6 = 128


def is_ip_range(input: str, version: Union[str, int] = None) -> bool:
    try:
        input = assert_string(input)

        parts = input.split('/')

        ip_part = String(parts[0])

        subnet_part = String(parts[1])

        if parts.length != 2:
            return False

        is_subnet_maybe = subnet_part.match(subnet_maybe)

        if not is_subnet_maybe:
            return False

        if subnet_part.length > 1 and subnet_part.starts_with('0'):
            return False

        is_valid_ip = is_ip(ip_part, version)

        if not is_valid_ip:
            return False

        expected_subnet = None
        if String(version) == '4':
            expected_subnet = subnet_v4
        elif String(version) == '6':
            expected_subnet = subnet_v6
        else:
            expected_subnet = subnet_v6 if is_ip(ip_part, '6') else subnet_v4

        return int(subnet_part) <= expected_subnet and int(subnet_part) >= 0
    except:
        return False



from typing import Any, Callable, TypeVar, Union

from ..includes import includes
from ..map import map
from ..reduce import reduce

T = TypeVar('T', bound='List')


class List(list):
    def map(self: T, cb) -> T:
        return List(map(self, cb))

    def reduce(self: T, cb, initializer=None) -> Union[str, int, float, bool, None]:
        return reduce(self, cb, initializer)

    @property
    def length(self: T) -> int:
        return len(self)

    def includes(self: T, val: Union[None, bool, str, int, float]) -> bool:
        return includes(self, val)

    def pop(self: T) -> Any:
        return list(self).pop()

    def join(self: T, join_with: str) -> str:
        return join_with.join(self)

    def find(self, cb: Callable[[Any, int], bool]) -> Union[Any, None]:
        for index, item in enumerate(self):
            if cb(item, index):
                return item
        return None



import inspect

# Storing as an object so that tests will continue to function properly if this string is ever changed
obj_str = '[object Object]'


def to_string(input):
    if isinstance(input, str):
        # in case a custom string type is passed, return as an actual str
        return str(input)
    elif isinstance(input, (int, float)):
        return str(input)
    elif isinstance(input, (dict, list)) or inspect.ismodule(input):
        return obj_str
    return ''



import unittest

from pyvalidator.is_btc_address import is_btc_address
from . import print_test_ok


class TestIsBtcAddress(unittest.TestCase):

    def test_valid_btc_address(self):
        for i in [
            '1MUz4VMYui5qY1mxUiG8BQ1Luv6tqkvaiL',
            '3J98t1WpEZ73CNmQviecrnyiWrnqRhWNLy',
            'bc1qar0srrr7xfkvy5l643lydnw9re59gtzzwf5mdq',
            '14qViLJfdGaP4EeHnDyJbEGQysnCpwk3gd',
            '35bSzXvRKLpHsHMrzb82f617cV4Srnt7hS',
            '17VZNX1SN5NtKa8UQFxwQbFeFc3iqRYhemt',
            'bc1qw508d6qejxtdg4y5r3zarvary0c5xw7kv8f3t4',
        ]:
            self.assertTrue(is_btc_address(i))
        print_test_ok()

    def test_invalid_btc_address(self):
        for i in [
            '4J98t1WpEZ73CNmQviecrnyiWrnqRhWNLy',
            '0x56F0B8A998425c53c75C4A303D4eF987533c5597',
            'pp8skudq3x5hzw8ew7vzsw8tn4k8wxsqsv0lt0mf3g',
            '17VZNX1SN5NlKa8UQFxwQbFeFc3iqRYhem',
            'BC1QW508D6QEJXTDG4Y5R3ZARVAYR0C5XW7KV8F3T4',
        ]:
            self.assertFalse(is_btc_address(i))
        print_test_ok()



import unittest

from pyvalidator.is_int import is_int
from . import print_test_ok


class TestIsInt(unittest.TestCase):

    def test_valid_ints(self):
        for i in [
            '13',
            '123',
            '0',
            '-0',
            '+1',
            '01',
            '-01',
            '000',
        ]:
            self.assertTrue(is_int(i))
        print_test_ok()

    def test_invalid_ints(self):
        for i in [
            '100e10',
            '123.123',
            '   ',
            '',
            '.',
            'foo',
        ]:
            self.assertFalse(is_int(i))
        print_test_ok()

    def test_valid_ints_dont_allow_leading_zeroes(self):
        for i in [
            '13',
            '11',
            '123',
            '0',
            '-0',
            '+1',
        ]:
            self.assertTrue(is_int(i, {"allow_leading_zeroes": False}))
        print_test_ok()

    def test_invalid_ints_dont_allow_leading_zeroes(self):
        for i in [
            '01',
            '-01',
            '000',
            '100e10',
            '123.11',
            'foo',
            '   ',
            '',
            '\n',
            '\t',
        ]:
            self.assertFalse(is_int(i, {"allow_leading_zeroes": False}))
        print_test_ok()

    def test_fail_ints(self):
        for i in [
            None,
            {},
        ]:
            self.assertRaises(Exception, is_int, i)
        print_test_ok()



import unittest

from pyvalidator.is_email import is_email
from . import print_test_ok


class TestIsEmail(unittest.TestCase):

    def test_valid_email(self):
        for i in [
            'foo@bar.com',
            'x@x.au',
            'foo@bar.com.au',
            'foo+bar@bar.com',
            'hans.m端ller@test.com',
            'hans@m端ller.com',
            'test|123@m端ller.com',
            'test123+ext@gmail.com',
            'some.name.midd.leNa.me.and.locality+extension@GoogleMail.com',
            '"foobar"@example.com',
            '"  foo  m端ller "@example.com',
            '"foo\\@bar"@example.com',
            "{}@{}.com".format(('a' * 64), ('a' * 63)),
            "{}@{}.com".format(('a' * 64), ('a' * 63)),
            "{}@gmail.com".format(('a' * 31)),
            'test@gmail.com',
            'test.1@gmail.com',
            'test@1337.com',
        ]:
            self.assertTrue(is_email(i))
        print_test_ok()

    def test_invalid_email(self):
        for i in [
            'invalidemail@',
            'invalid.com',
            '@invalid.com',
            'foo@bar.com.',
            'somename@ｇｍａｉｌ.com',
            'foo@bar.co.uk.',
            'z@co.c',
            'ｇｍａｉｌｇｍａｉｌｇｍａｉｌｇｍａｉｌｇｍａｉｌ@gmail.com',
            "{}@{}.com".format('a' * 64, 'a' * 251),
            "{}@{}.com".format('a' * 65, 'a' * 250),
            "{}@{}.com".format('a' * 64, 'a' * 64),
            "{}@{}.{}.{}.{}.com".format('a' * 64, 'b' * 63, 'b' * 63, 'c' * 63, 'd' * 58),
            'test1@invalid.co m',
            'test2@invalid.co m',
            'test3@invalid.co m',
            'test4@invalid.co m',
            'test5@invalid.co m',
            'test6@invalid.co m',
            'test7@invalid.co m',
            'test8@invalid.co m',
            'test9@invalid.co m',
            'test10@invalid.co m',
            'test11@invalid.co m',
            'test12@invalid.co　m',
            'test13@invalid.co　m',
            'multiple..dots@stillinvalid.com',
            'test123+invalid! sub_address@gmail.com',
            'gmail...ignores...dots...@gmail.com',
            'ends.with.dot.@gmail.com',
            'multiple..dots@gmail.com',
            'wrong()[]",:;<>@@gmail.com',
            '"wrong()[]",:;<>@@gmail.com',
            'username@domain.com�',
            'username@domain.com©',
        ]:
            self.assertFalse(is_email(i))
        print_test_ok()

    def test_valid_domain_specific_email(self):
        for i in [
            'foobar@gmail.com',
            'foo.bar@gmail.com',
            'foo.bar@googlemail.com',
            '{}@gmail.com'.format('a' * 30),
        ]:
            self.assertTrue(is_email(i, {"domain_specific_validation": True}))
        print_test_ok()

    def test_invalid_domain_specific_email(self):
        for i in [
            'test.1@gmail.com',
            '.foobar@gmail.com',
            "{}@gmail.com".format('s' * 31),
        ]:
            self.assertFalse(is_email(i, {"domain_specific_validation": True}))
        print_test_ok()

    def test_valid_email_with_utf8_char_in_local_part(self):
        for i in [
            'foo@bar.com',
            'x@x.au',
            'foo@bar.com.au',
            'foo+bar@bar.com',
            'hans@m端ller.com',
            'test|123@m端ller.com',
            'test123+ext@gmail.com',
            'some.name.midd.leNa.me+extension@GoogleMail.com',
            '"foobar"@example.com',
            '"foo\\@bar"@example.com',
            '"  foo  bar  "@example.com',
        ]:
            self.assertTrue(is_email(i, {"allow_utf8_local_part": False}))
        print_test_ok()

    def test_invalid_email_with_utf8_char_in_local_part(self):
        for i in [
            'invalidemail@',
            'invalid.com',
            '@invalid.com',
            'foo@bar.com.',
            'foo@bar.co.uk.',
            'somename@ｇｍａｉｌ.com',
            'hans.m端ller@test.com',
            'z@co.c',
            'tüst@invalid.com',
        ]:
            self.assertFalse(is_email(i, {"allow_utf8_local_part": False}))
        print_test_ok()

    def test_valid_email_with_display_name(self):
        for i in [
            'foo@bar.com',
            'x@x.au',
            'foo@bar.com.au',
            'foo+bar@bar.com',
            'hans.m端ller@test.com',
            'hans@m端ller.com',
            'test|123@m端ller.com',
            'test123+ext@gmail.com',
            'some.name.midd.leNa.me+extension@GoogleMail.com',
            'Some Name <foo@bar.com>',
            'Some Name <x@x.au>',
            'Some Name <foo@bar.com.au>',
            'Some Name <foo+bar@bar.com>',
            'Some Name <hans.m端ller@test.com>',
            'Some Name <hans@m端ller.com>',
            'Some Name <test|123@m端ller.com>',
            'Some Name <test123+ext@gmail.com>',
            '\'Foo Bar, Esq\'<foo@bar.com>',
            'Some Name <some.name.midd.leNa.me+extension@GoogleMail.com>',
            'Some Middle Name <some.name.midd.leNa.me+extension@GoogleMail.com>',
            'Name <some.name.midd.leNa.me+extension@GoogleMail.com>',
            'Name<some.name.midd.leNa.me+extension@GoogleMail.com>',
            'Some Name <foo@gmail.com>',
            'Name🍓With🍑Emoji🚴‍♀️🏆<test@aftership.com>',
            '🍇🍗🍑<only_emoji@aftership.com>',
            '"<displayNameInBrackets>"<jh@gmail.com>',
            '"\\"quotes\\""<jh@gmail.com>',
            '"name;"<jh@gmail.com>',
            '"name;" <jh@gmail.com>',
        ]:
            self.assertTrue(is_email(i, {"allow_display_name": True}))
        print_test_ok()

    def test_invalid_email_with_display_name(self):
        for i in [
            'invalidemail@',
            'invalid.com',
            '@invalid.com',
            'foo@bar.com.',
            'foo@bar.co.uk.',
            'Some Name <invalidemail@>',
            'Some Name <invalid.com>',
            'Some Name <@invalid.com>',
            'Some Name <foo@bar.com.>',
            'Some Name <foo@bar.co.uk.>',
            'Some Name foo@bar.co.uk.>',
            'Some Name <foo@bar.co.uk.',
            'Some Name < foo@bar.co.uk >',
            'Name foo@bar.co.uk',
            'Some Name <some..name@gmail.com>',
            'Some Name<emoji_in_address🍈@aftership.com>',
            'invisibleCharacter\u001F<jh@gmail.com>',
            '<displayNameInBrackets><jh@gmail.com>',
            '\\"quotes\\"<jh@gmail.com>',
            '""quotes""<jh@gmail.com>',
            'name;<jh@gmail.com>',
            '    <jh@gmail.com>',
            '"    "<jh@gmail.com>',
        ]:
            self.assertFalse(is_email(i, {"allow_display_name": True}))
        print_test_ok()

    def test_valid_email_with_required_display_name(self):
        for i in [
            'Some Name <foo@bar.com>',
            'Some Name <x@x.au>',
            'Some Name <foo@bar.com.au>',
            'Some Name <foo+bar@bar.com>',
            'Some Name <hans.m端ller@test.com>',
            'Some Name <hans@m端ller.com>',
            'Some Name <test|123@m端ller.com>',
            'Some Name <test123+ext@gmail.com>',
            'Some Name <some.name.midd.leNa.me+extension@GoogleMail.com>',
            'Some Middle Name <some.name.midd.leNa.me+extension@GoogleMail.com>',
            'Name <some.name.midd.leNa.me+extension@GoogleMail.com>',
            'Name<some.name.midd.leNa.me+extension@GoogleMail.com>',
        ]:
            self.assertTrue(is_email(i, {"require_display_name": True}))
        print_test_ok()

    def test_invalid_email_with_required_display_name(self):
        for i in [
            'some.name.midd.leNa.me+extension@GoogleMail.com',
            'foo@bar.com',
            'x@x.au',
            'foo@bar.com.au',
            'foo+bar@bar.com',
            'hans.m端ller@test.com',
            'hans@m端ller.com',
            'test|123@m端ller.com',
            'test123+ext@gmail.com',
            'invalidemail@',
            'invalid.com',
            '@invalid.com',
            'foo@bar.com.',
            'foo@bar.co.uk.',
            'Some Name <invalidemail@>',
            'Some Name <invalid.com>',
            'Some Name <@invalid.com>',
            'Some Name <foo@bar.com.>',
            'Some Name <foo@bar.co.uk.>',
            'Some Name foo@bar.co.uk.>',
            'Some Name <foo@bar.co.uk.',
            'Some Name < foo@bar.co.uk >',
            'Name foo@bar.co.uk',
        ]:
            self.assertFalse(is_email(i, {"require_display_name": True}))
        print_test_ok()

    def test_valid_emails_with_ip(self):
        for i in [
            'email@[123.123.123.123]',
            'email@255.255.255.255',
        ]:
            self.assertTrue(is_email(i, {"allow_ip_domain": True}))
        print_test_ok()

    def test_invalid_emails_with_ip(self):
        for i in [
            'email@0.0.0.256',
            'email@26.0.0.256',
            'email@[266.266.266.266]',
        ]:
            self.assertFalse(is_email(i, {"allow_ip_domain": True}))
        print_test_ok()

    def test_valid_emails_with_blacklisted_chars_in_name(self):
        for i in [
            ['emil@gmail.com', {"blacklisted_chars": 'abc'}],
            ['ssss@gmail.com', {"blacklisted_chars": 'abc'}],
        ]:
            self.assertTrue(is_email(*i))
        for i in [
            ['today@yahoo.com', {"blacklisted_chars": 'time'}],
        ]:
            self.assertFalse(is_email(*i))
        print_test_ok()

    def test_invalid_emails_with_blacklisted_chars_in_name(self):
        for i in [
            ['email@gmail.com', {"blacklisted_chars": 'abc'}],
            ['ssss@gmail.com', {"blacklisted_chars": 's'}],
            ['ssss@gmail.com', {"blacklisted_chars": 'ssss'}],
            ['today@yahoo.com', {"blacklisted_chars": 'day'}],
        ]:
            self.assertFalse(is_email(*i))
        print_test_ok()

    def test_valid_long_emails(self):
        for i in [
            'Deleted-user-id-19430-Team-5051deleted-user-id-19430-team-5051XXXXXX@example.com',
        ]:
            self.assertTrue(is_email(i, {"ignore_max_length": True}))
        print_test_ok()

    def test_invalid_long_emails(self):
        for i in [
            'Deleted-user-id-19430-Team-5051deleted-user-id-19430-team-5051XXXXXX@example.com',
            'Deleted-user-id-19430-Team-5051deleted-user-id-19430-team-5051XXXXXX-blablabla-blablabla-blablabla-blablabla-blablabla-blablabla@example.com',
        ]:
            self.assertFalse(is_email(i, {"ignore_max_length": False}))
        print_test_ok()

    def test_valid_emails_with_denied_domains(self):
        for i in [
            ['sample@test.com', {"host_blacklist": ['web.com', 'test.domain.com']}],
            ['sample@domain.com', {"host_blacklist": ['web.com', 'test.domain.com']}],
        ]:
            self.assertTrue(is_email(*i))
        print_test_ok()

    def test_invalid_emails_with_denied_domains(self):
        for i in [
            ['gosn@web.com', {"host_blacklist": ['web.com', 'test.domain.com']}],
            ['gosn@test.domain.com', {"host_blacklist": ['web.com', 'test.domain.com']}],
        ]:
            self.assertFalse(is_email(*i))
        print_test_ok()



from typing import Union

from .utils.to_int import to_int


def is_even(input: Union[int, str]) -> bool:
    input = to_int(input)

    if input is None:
        return False

    return not bool(input % 2)



import unittest

from pyvalidator.is_json import is_json
from . import print_test_ok


class TestIsJson(unittest.TestCase):

    def test_valid_json(self):
        self.assertTrue(is_json('{ "key": "value" }'))
        self.assertTrue(is_json('{}'))
        print_test_ok()

    def test_invalid_json(self):
        self.assertFalse(is_json('{ key: "value" }'))
        self.assertFalse(is_json('{ \'key\': \'value\' }'))
        self.assertFalse(is_json('null'))
        self.assertFalse(is_json('1234'))
        self.assertFalse(is_json('11.1'))
        self.assertFalse(is_json('nope'))
        self.assertFalse(is_json('None'))
        self.assertFalse(is_json('false'))
        self.assertFalse(is_json('true'))
        print_test_ok()

    def test_valid_json_allow_primitives(self):
        self.assertTrue(is_json('{ "key": "value" }', { "allow_primitives": True }))
        self.assertTrue(is_json('{}', { "allow_primitives": True }))
        self.assertTrue(is_json('false', { "allow_primitives": True }))
        self.assertTrue(is_json('true', { "allow_primitives": True }))
        print_test_ok()

    def test_invalid_json_allow_primitives(self):
        self.assertFalse(is_json('{ key: "value" }', { "allow_primitives": True }))
        self.assertFalse(is_json('{ \'key\': \'value\' }', { "allow_primitives": True }))
        self.assertFalse(is_json('{ "key": value }', { "allow_primitives": True }))
        self.assertFalse(is_json('1234', { "allow_primitives": True }))
        self.assertFalse(is_json('nope', { "allow_primitives": True }))
        self.assertFalse(is_json('None', { "allow_primitives": True }))
        print_test_ok()



import unittest

from pyvalidator.is_port import is_port
from . import print_test_ok


class TestIsPort(unittest.TestCase):

    def test_valid_port(self):
        for i in [
            '0',
            '22',
            '80',
            '443',
            '5000',
            '8000',
            '65535',
        ]:
            self.assertTrue(is_port(i))
        print_test_ok()

    def test_invalid_port(self):
        for i in [
            '',
            '-1',
            '65536',
            '655369',
            '.',
        ]:
            self.assertFalse(is_port(i))
        print_test_ok()



import unittest

from pyvalidator.is_url import is_url
from . import print_test_ok


class TestIsUrl(unittest.TestCase):
    def valid_check(self, items, options = {}):
        for item in items:
            try:
                self.assertTrue(is_url(item, options))
            except Exception as e:
                print(f'failed for input: {item}')
                raise e

    def invalid_check(self, items, options = {}):
        for item in items:
            try:
                self.assertFalse(is_url(item, options))
            except Exception as e:
                print(f'failed for input: {item}')
                raise e

    def test_valid_url(self):
        valid_items = [
            'https://stackoverflow.com/',
            'www.google.com',
            'HTTPS://stackoverflow.com/',
            'www.GOOGLE.com',
            'facebook.com',
            'facebook.COM',
            'http://rs.linkedin.com',
            'test.net',
            'unknowns.cc/yes/please',
            'worldofwarcraft.com/en-gb/game?races=blood-elf',
            'https://seeklogo.com/images/P/pearl-jam-alive-logo-8FA34991E4-seeklogo.com.png',
            'http://api.google.com/q?exp=a%7Cb',
            'https://companydomain.bamboohr.com/authorize.php?request=authorize&state=state&response_type=code&scope=openid&client_id=client_id&redirect_uri=redirecturl',
            'https://id.b2b.yahooinc.com/identity/oauth2/access_token?realm=true_story',
            'https://api.twitter.com/1.1/search/tweets.json',
            'https://api.twitter.com/1.1/search/%20/tweets.json',
            'http://foo.com/blahblah_(wikipedia)',
            'https://test-domain.com/',
            'ha.com',
            'www.go.abcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyzaaaaaaaaaaa'
        ]
        self.valid_check(valid_items)
        print_test_ok()

    def test_invalid_url(self):
        invalid_items = [
            '',
            '.',
            'helloworld',
            '127.0.0.1',
            'ffs://silvermoon.com',
            'ffs:/silvermoon.com',
            '/some/random/path',
            'C:\\Documents',
            's.s',
            'http://mw1.google.com/mw-earth-vectordb/kml-samples/gp/seattle/gigapxl/$[level]/r$[y]_c$[x].jpg',
            'http://api.google.com/q?exp=a|b',
            'https://id.b2b.yahooinc.com/identity/oauth2/access_token?realm=true story',
            'https://api.twitter.com/1.1/search/ /tweets.json',
            'http://-test-domain.com',
            'http://test-domain-.com',
            'http://test_domain.com',
            'http://test-domain_.com',
            'http://_test-domain.com',
            'http://test*domain.com',
            'http://test!domain.com',
            'http://test-domain!.com',
            'http://test-domain*.com',
            'hello+world.io',
            'hello=world.io',
            'hello@world.io',
            'hello%world.io',
            'hello:world.io',
            'hello~world.io',
            'h.com',
            'www.go.abcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyzaaaaaaaaaaaa'
        ]
        self.invalid_check(invalid_items)
        print_test_ok()

    def test_valid_url_with_target_domains(self):
        valid_items = [
            'https://stackoverflow.com/',
            'www.google.com',
            'http://api.google.com/q?exp=a%7Cb',
        ]
        self.valid_check(valid_items, { "domains": [ "stackoverflow", "google" ] })
        print_test_ok()

    def test_valid_url_with_target_domain(self):
        valid_items = [
            'https://stackoverflow.com/'
        ]
        self.valid_check(valid_items, { "domains": [ "stackoverflow" ] })
        print_test_ok()

    def test_invalid_url_with_target_domains(self):
        invalid_items = [
            'https://stackoverflow.com/',
            'www.google.com',
            'http://api.google.com/q?exp=a%7Cb',
        ]
        self.invalid_check(invalid_items, { "domains": [ "facebook", "instagram" ] })
        print_test_ok()

    def test_invalid_url_with_target_domain(self):
        invalid_items = [
            'https://stackoverflow.com/',
            'www.google.com',
            'http://api.google.com/q?exp=a%7Cb'
        ]
        self.invalid_check(invalid_items, { "domains": [ "twitter" ] })
        print_test_ok()

    def test_valid_url_with_top_level_domains(self):
        items = [
            'https://stackoverflow.com/',
            'www.google.com',
            'en.wikipedia.org'
        ]
        self.valid_check(items, { "top_level_domains": [ "com", "org" ] })
        print_test_ok()

    def test_invalid_url_with_top_level_domains(self):
        items = [
            'https://stackoverflow.com/',
            'www.google.com'
        ]
        self.invalid_check(items, { "top_level_domains": [ "net" ] })
        print_test_ok()

    def test_invalid_case_sensitive_url(self):
        items = [
            'HTTPS://stackoverflow.com/',
            'www.google.COM',
            'tWITTer.com'
        ]
        self.invalid_check(items, { "insensitive": False })
        print_test_ok()

    def test_valid_url_with_no_scheme(self):
        items = [
            'en.wikipedia.org/',
            'www.youtube.com'
        ]
        self.valid_check(items, { "no_scheme": True })
        print_test_ok()

    def test_invalid_url_with_no_scheme(self):
        items = [
            'https://en.wikipedia.org/',
            'http://www.youtube.com'
        ]
        self.invalid_check(items, { "no_scheme": True })
        print_test_ok()

    def test_valid_url_with_no_path(self):
        items = [
            'https://en.wikipedia.org/',
            'https://www.youtube.com'
        ]
        self.valid_check(items, { "with_no_path": True })
        print_test_ok()

    def test_invalid_url_for_including_a_path(self):
        items = [
            'https://en.wikipedia.org/wiki/Grunge',
            'https://www.youtube.com/watch?v=iXjMbdHa-IU'
        ]
        self.invalid_check(items, { "with_no_path": True })
        print_test_ok()



import unittest

from pyvalidator.is_fqdn import is_fqdn
from . import print_test_ok


class TestIsFqdn(unittest.TestCase):

    def test_valid_fqdn(self):
        for i in [
            'domain.com',
            'dom.plato',
            'a.domain.co',
            'foo--bar.com',
            'xn--froschgrn-x9a.com',
            'rebecca.blackfriday',
            '1337.com',
        ]:
            self.assertTrue(is_fqdn(i))
        print_test_ok()

    def test_invalid_fqdn(self):
        for i in [
            'abc',
            '256.0.0.0',
            '_.com',
            '*.some.com',
            's!ome.com',
            'domain.com/',
            '/more.com',
            'domain.com�',
            'domain.co\u00A0m',
            'domain.co\u1680m',
            'domain.co\u2006m',
            'domain.co\u2028m',
            'domain.co\u2029m',
            'domain.co\u202Fm',
            'domain.co\u205Fm',
            'domain.co\u3000m',
            'domain.com\uDC00',
            'domain.co\uEFFFm',
            'domain.co\uFDDAm',
            'domain.co\uFFF4m',
            'domain.com©',
            'example.0',
            '192.168.0.9999',
            '192.168.0',
        ]:
            self.assertFalse(is_fqdn(i))
        print_test_ok()

    def test_valid_fqdn_with_trailing_dot(self):
        for i in [
            'domain.com.',
        ]:
            self.assertTrue(is_fqdn(i, {"allow_trailing_dot": True}))
        print_test_ok()

    def test_invalid_fqdn_with_not_required_tld(self):
        for i in [
            'example.0',
            '192.168.0',
            '192.168.0.9999',
        ]:
            self.assertFalse(is_fqdn(i, {"require_tld": False}))
        print_test_ok()

    def test_valid_fqdn_with_allowed_wildcard(self):
        for i in [
            '*.domain.com',
            '*.global.domain.com',
        ]:
            self.assertTrue(is_fqdn(i, {"allow_wildcard": True}))
        print_test_ok()

    def test_valid_fqdn_with_not_required_tld_but_allowed_numeric_tld(self):
        for i in [
            'example.0',
            '192.168.0',
            '192.168.0.9999',
        ]:
            self.assertTrue(is_fqdn(i, {"require_tld": False, "allow_numeric_tld": True}))
        print_test_ok()



import datetime

import timestring

from .assert_string import assert_string


def __parse_date(input: str):
    try:
        return timestring.Date(input).date
    except:
        return None


def to_date(input: str):
    assert_string(input)
    date = __parse_date(input)
    return date if bool(date) else None


def default_date_string():
    return datetime.datetime.now().isoformat()



from .utils.assert_string import assert_string


def is_license_plate(input: str, locale: str = 'any') -> bool:
    input = assert_string(input)

    patterns = {
        'cs-CZ': r"^(([ABCDEFHKIJKLMNPRSTUVXYZ]|[0-9])-?){5,8}$",
        'de-DE': r"^((A|AA|AB|AC|AE|AH|AK|AM|AN|AÖ|AP|AS|AT|AU|AW|AZ|B|BA|BB|BC|BE|BF|BH|BI|BK|BL|BM|BN|BO|BÖ|BS|BT|BZ|C|CA|CB|CE|CO|CR|CW|D|DA|DD|DE|DH|DI|DL|DM|DN|DO|DU|DW|DZ|E|EA|EB|ED|EE|EF|EG|EH|EI|EL|EM|EN|ER|ES|EU|EW|F|FB|FD|FF|FG|FI|FL|FN|FO|FR|FS|FT|FÜ|FW|FZ|G|GA|GC|GD|GE|GF|GG|GI|GK|GL|GM|GN|GÖ|GP|GR|GS|GT|GÜ|GV|GW|GZ|H|HA|HB|HC|HD|HE|HF|HG|HH|HI|HK|HL|HM|HN|HO|HP|HR|HS|HU|HV|HX|HY|HZ|IK|IL|IN|IZ|J|JE|JL|K|KA|KB|KC|KE|KF|KG|KH|KI|KK|KL|KM|KN|KO|KR|KS|KT|KU|KW|KY|L|LA|LB|LC|LD|LF|LG|LH|LI|LL|LM|LN|LÖ|LP|LR|LU|M|MA|MB|MC|MD|ME|MG|MH|MI|MK|ML|MM|MN|MO|MQ|MR|MS|MÜ|MW|MY|MZ|N|NB|ND|NE|NF|NH|NI|NK|NM|NÖ|NP|NR|NT|NU|NW|NY|NZ|OA|OB|OC|OD|OE|OF|OG|OH|OK|OL|OP|OS|OZ|P|PA|PB|PE|PF|PI|PL|PM|PN|PR|PS|PW|PZ|R|RA|RC|RD|RE|RG|RH|RI|RL|RM|RN|RO|RP|RS|RT|RU|RV|RW|RZ|S|SB|SC|SE|SG|SI|SK|SL|SM|SN|SO|SP|SR|ST|SU|SW|SY|SZ|TE|TF|TG|TO|TP|TR|TS|TT|TÜ|ÜB|UE|UH|UL|UM|UN|V|VB|VG|VK|VR|VS|W|WA|WB|WE|WF|WI|WK|WL|WM|WN|WO|WR|WS|WT|WÜ|WW|WZ|Z|ZE|ZI|ZP|ZR|ZW|ZZ)[- ]?[A-Z]{1,2}[- ]?\d{1,4}|(ABG|ABI|AIB|AIC|ALF|ALZ|ANA|ANG|ANK|APD|ARN|ART|ASL|ASZ|AUR|AZE|BAD|BAR|BBG|BCH|BED|BER|BGD|BGL|BID|BIN|BIR|BIT|BIW|BKS|BLB|BLK|BNA|BOG|BOH|BOR|BOT|BRA|BRB|BRG|BRK|BRL|BRV|BSB|BSK|BTF|BÜD|BUL|BÜR|BÜS|BÜZ|CAS|CHA|CLP|CLZ|COC|COE|CUX|DAH|DAN|DAU|DBR|DEG|DEL|DGF|DIL|DIN|DIZ|DKB|DLG|DON|DUD|DÜW|EBE|EBN|EBS|ECK|EIC|EIL|EIN|EIS|EMD|EMS|ERB|ERH|ERK|ERZ|ESB|ESW|FDB|FDS|FEU|FFB|FKB|FLÖ|FOR|FRG|FRI|FRW|FTL|FÜS|GAN|GAP|GDB|GEL|GEO|GER|GHA|GHC|GLA|GMN|GNT|GOA|GOH|GRA|GRH|GRI|GRM|GRZ|GTH|GUB|GUN|GVM|HAB|HAL|HAM|HAS|HBN|HBS|HCH|HDH|HDL|HEB|HEF|HEI|HER|HET|HGN|HGW|HHM|HIG|HIP|HMÜ|HOG|HOH|HOL|HOM|HOR|HÖS|HOT|HRO|HSK|HST|HVL|HWI|IGB|ILL|JÜL|KEH|KEL|KEM|KIB|KLE|KLZ|KÖN|KÖT|KÖZ|KRU|KÜN|KUS|KYF|LAN|LAU|LBS|LBZ|LDK|LDS|LEO|LER|LEV|LIB|LIF|LIP|LÖB|LOS|LRO|LSZ|LÜN|LUP|LWL|MAB|MAI|MAK|MAL|MED|MEG|MEI|MEK|MEL|MER|MET|MGH|MGN|MHL|MIL|MKK|MOD|MOL|MON|MOS|MSE|MSH|MSP|MST|MTK|MTL|MÜB|MÜR|MYK|MZG|NAB|NAI|NAU|NDH|NEA|NEB|NEC|NEN|NES|NEW|NMB|NMS|NOH|NOL|NOM|NOR|NVP|NWM|OAL|OBB|OBG|OCH|OHA|ÖHR|OHV|OHZ|OPR|OSL|OVI|OVL|OVP|PAF|PAN|PAR|PCH|PEG|PIR|PLÖ|PRÜ|QFT|QLB|RDG|REG|REH|REI|RID|RIE|ROD|ROF|ROK|ROL|ROS|ROT|ROW|RSL|RÜD|RÜG|SAB|SAD|SAN|SAW|SBG|SBK|SCZ|SDH|SDL|SDT|SEB|SEE|SEF|SEL|SFB|SFT|SGH|SHA|SHG|SHK|SHL|SIG|SIM|SLE|SLF|SLK|SLN|SLS|SLÜ|SLZ|SMÜ|SOB|SOG|SOK|SÖM|SON|SPB|SPN|SRB|SRO|STA|STB|STD|STE|STL|SUL|SÜW|SWA|SZB|TBB|TDO|TET|TIR|TÖL|TUT|UEM|UER|UFF|USI|VAI|VEC|VER|VIB|VIE|VIT|VOH|WAF|WAK|WAN|WAR|WAT|WBS|WDA|WEL|WEN|WER|WES|WHV|WIL|WIS|WIT|WIZ|WLG|WMS|WND|WOB|WOH|WOL|WOR|WOS|WRN|WSF|WST|WSW|WTL|WTM|WUG|WÜM|WUN|WUR|WZL|ZEL|ZIG)[- ]?(([A-Z][- ]?\d{1,4})|([A-Z]{2}[- ]?\d{1,3})))[- ]?(E|H)?$",
        'de-LI': r"^FL[- ]?\d{1,5}[UZ]?$",
        'fi-FI': r"^(?=.{4,7})(([A-Z]{1,3}|[0-9]{1,3})[\s-]?([A-Z]{1,3}|[0-9]{1,5}))$",
        'pt-PT': r"^([A-Z]{2}|[0-9]{2})[ -·]?([A-Z]{2}|[0-9]{2})[ -·]?([A-Z]{2}|[0-9]{2})$",
        'sq-AL': r"^[A-Z]{2}[- ]?((\d{3}[- ]?(([A-Z]{2})|T))|(R[- ]?\d{3}))$",
        'pt-BR': r"^[A-Z]{3}[ -]?[0-9][A-Z][0-9]{2}|[A-Z]{3}[ -]?[0-9]{4}$",
        'sv-SE': r"^[A-HJ-PR-UW-Z]{3} ?[\d]{2}[A-HJ-PR-UW-Z1-9]$|(^[A-ZÅÄÖ ]{2,7}$)/.test(str.trim())",
        'en-IN': r"^[A-Z]{2}[ -]?[0-9]{1,2}(?:[ -]?[A-Z])(?:[ -]?[A-Z]*)?[ -]?[0-9]{4}$",
        'sr-RS': r"(^(AL|AR|AC|BG|BO|BP|BT|BĆ|BU|BČ|VA|VB|VL|VP|VR|VS|VŠ|GL|GM|DE|ĐA|ZA|ZR|IN|IC|JA|KA|KV|KG|KŽ|KI|KL|KM|KO|KŠ|LB|LE|LO|LU|NV|NG|NI|NP|NS|PA|PB|PE|PŽ|PZ|PI|PK|PN|PO|PP|PR|PT|RA|RU|SA|SV|SD|SM|SO|SP|ST|SU|TO|TS|TT|ĆU|UB|UE|UR|ČA|ŠA|ŠI)[- ]?\d{3,4}[- ]?[A-ZČĆĐŠŽ]{2}$)|(^П[- ]?\d{3}[- ]?\d{3}$)|(^(AL|AR|AC|BG|BO|BP|BT|BĆ|BU|BČ|VA|VB|VL|VP|VR|VS|VŠ|GL|GM|DE|ĐA|ZA|ZR|IN|IC|JA|KA|KV|KG|KŽ|KI|KL|KM|KO|KŠ|LB|LE|LO|LU|NV|NG|NI|NP|NS|PA|PB|PE|PŽ|PZ|PI|PK|PN|PO|PP|PR|PT|RA|RU|SA|SV|SD|SM|SO|SP|ST|SU|TO|TS|TT|ĆU|UB|UE|UR|ČA|ŠA|ŠI)[- ]?\d{2}[- ]?\d{3}$)"
    }

    patterns['hi-IN'] = patterns['en-IN']
    patterns['gu-IN'] = patterns['en-IN']
    patterns['as-IN'] = patterns['en-IN']
    patterns['bn-IN'] = patterns['en-IN']
    patterns['kn-IN'] = patterns['en-IN']
    patterns['ml-IN'] = patterns['en-IN']
    patterns['mr-IN'] = patterns['en-IN']
    patterns['or-IN'] = patterns['en-IN']
    patterns['pa-IN'] = patterns['en-IN']
    patterns['sa-IN'] = patterns['en-IN']
    patterns['ta-IN'] = patterns['en-IN']
    patterns['te-IN'] = patterns['en-IN']
    patterns['kok-IN'] = patterns['en-IN']


    if locale not in patterns and locale != 'any':
        raise ValueError('Locale is not valid: ', locale)

    if locale in patterns:
        return input.match(patterns[locale])
    else:
        for patternKey in patterns:
            matches = input.match(patterns[patternKey])
            if matches:
                return True
        return False
    
    



import unittest

from pyvalidator import *
from . import print_test_ok


class TestIsHtmlElement(unittest.TestCase):

    def test_valid_html_element(self):
        for i in [
            '<div></div>',
            '<div ></ div>',
            '<    div ></ div>',
            '<div class="container"></div>',
            '<a></a>',
            '<a href=""></a>',
            '<input type="password" />',
            '<input disabled />',
            '<!DOCTYPE>',
            '<h1></h1>',
            '<h2></h2>',
            '<h3></h3>',
            '<h4></h4>',
            '<h5></h5>',
            '<h6>text</h6>',
            '<body><div></div></body>',
            '<base href="https://www.w3schools.com/" target="_blank">',
            '<img />',
            '<head></head>',
            '<label for="cars">Choose a car:</label>',
            '<p>&amp;</p>',
            '<textarea id="w3review" name="w3review" rows="4" cols="50"></textarea>',

            """
                        <style>
                            h1 {color:red;}
                            p {color:blue;}
                        </style>
                    """,

            """
                        <!DOCTYPE html>
                        <html>
                            <head>
                                <title>Page Title</title>
                            </head>
                            <body>
                                <h1>This is a Heading</h1>
                                <p>This is a paragraph.</p>
                            </body>
                        </html>
                    """,
        ]:
            self.assertTrue(is_html_element(i))
        print_test_ok()

    def test_invalid_html_element(self):
        for i in [
            '<></>',
            '<<>></>',
            '<>',
            '<random></random>',
            '<body><body>',
            '0',
            '<h2></h1>',
            '<h3></h1>',
            '<h4></h1>',
            '<h5></h1>',
            '<h6>',
            '</h6>',
            '<h6></h6><h3></h3>',
            '<head>',
            '<img',
            '<img <>',
            '<<img <>',
            '<image />',
            '<image />',
            '<input></input>',
            '</textarea>',
            '</span>',
            '</span>',
            '<html lang="en">',
            '.',
            '',
            ' ',
            '<p>paragraph</p> some text after',
            'some text before <p>paragraph</p>',

            """
                        <style>
                            h1 {color:red;}
                            p {color:blue;}
                        <style>
                    """

        ]:
            self.assertFalse(is_html_element(i))
        print_test_ok()

    def test_valid_html_element_contains(self):
        for i in [
            'some text <p>paragraph</p>, exactly.',
            'some text <br>brake, exactly.',
            'some text <input /> input, exactly.',
        ]:
            self.assertTrue(is_html_element(i, {"contains": True}))
        print_test_ok()



import unittest

from pyvalidator.is_data_uri import is_data_uri
from . import print_test_ok


class TestIsDataUri(unittest.TestCase):

    def test_valid_data_uri(self):
        for i in [
            'data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQAQMAAAAlPW0iAAAABlBMVEUAAAD///+l2Z/dAAAAM0lEQVR4nGP4/5/h/1+G/58ZDrAz3D/McH8yw83NDDeNGe4Ug9C9zwz3gVLMDA/A6P9/AFGGFyjOXZtQAAAAAElFTkSuQmCC',
            'data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAgAAAAIBAMAAAA2IaO4AAAAFVBMVEXk5OTn5+ft7e319fX29vb5+fn///++GUmVAAAALUlEQVQIHWNICnYLZnALTgpmMGYIFWYIZTA2ZFAzTTFlSDFVMwVyQhmAwsYMAKDaBy0axX/iAAAAAElFTkSuQmCC',
            '   data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAgAAAAIBAMAAAA2IaO4AAAAFVBMVEXk5OTn5+ft7e319fX29vb5+fn///++GUmVAAAALUlEQVQIHWNICnYLZnALTgpmMGYIFWYIZTA2ZFAzTTFlSDFVMwVyQhmAwsYMAKDaBy0axX/iAAAAAElFTkSuQmCC   ',
            'data:image/svg+xml;charset=utf-8,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%22100%22%20height%3D%22100%22%3E%3Crect%20fill%3D%22%2300B1FF%22%20width%3D%22100%22%20height%3D%22100%22%2F%3E%3C%2Fsvg%3E',
            'data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxMDAiIGhlaWdodD0iMTAwIj48cmVjdCBmaWxsPSIjMDBCMUZGIiB3aWR0aD0iMTAwIiBoZWlnaHQ9IjEwMCIvPjwvc3ZnPg==',
            ' data:,Hello%2C%20World!',
            ' data:,Hello World!',
            ' data:text/plain;base64,SGVsbG8sIFdvcmxkIQ%3D%3D',
            ' data:text/html,%3Ch1%3EHello%2C%20World!%3C%2Fh1%3E',
            'data:,A%20brief%20note',
            'data:text/html;charset=US-ASCII,%3Ch1%3EHello!%3C%2Fh1%3E',
        ]:
            self.assertTrue(is_data_uri(i))
        print_test_ok()

    def test_invalid_data_uri(self):
        for i in [
            'dataxbase64',
            'data:HelloWorld',
            'data:,A%20brief%20invalid%20[note',
            'file:text/plain;base64,SGVsbG8sIFdvcmxkIQ%3D%3D',
            'data:text/html;charset=,%3Ch1%3EHello!%3C%2Fh1%3E',
            'data:text/html;charset,%3Ch1%3EHello!%3C%2Fh1%3E',
            'data:base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQAQMAAAAlPW0iAAAABlBMVEUAAAD///+l2Z/dAAAAM0lEQVR4nGP4/5/h/1+G/58ZDrAz3D/McH8yw83NDDeNGe4Ug9C9zwz3gVLMDA/A6P9/AFGGFyjOXZtQAAAAAElFTkSuQmCC',
            '',
            'http://wikipedia.org',
            'base64',
            'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQAQMAAAAlPW0iAAAABlBMVEUAAAD///+l2Z/dAAAAM0lEQVR4nGP4/5/h/1+G/58ZDrAz3D/McH8yw83NDDeNGe4Ug9C9zwz3gVLMDA/A6P9/AFGGFyjOXZtQAAAAAElFTkSuQmCC',
        ]:
            self.assertFalse(is_data_uri(i))
        print_test_ok()



from setuptools import setup

with open('README.md', encoding="utf8") as f:
    readme = f.read()

MAJOR = 0
MINOR = 17
PATCH = 0

VERSION = '{}.{}.{}'.format(MAJOR, MINOR, PATCH)
DESCRIPTION = 'String validation and sanitization'

# Setting up
setup(
    name="py.validator",
    version=VERSION,
    author="Sanel",
    author_email="sanelbgd@hotmail.com",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=readme,
    packages=['pyvalidator', 'pyvalidator.utils', 'pyvalidator.utils.Classes'],
    install_requires=['typing', 'timestring'],
    keywords=['python', 'validation', 'string', 'sanitization', 'validator.js'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3.8",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ],
    project_urls={
        'GitHub': 'https://github.com/theteladras/py.validator'
    },
    license='MIT'
)



import unittest

from pyvalidator.is_odd import is_odd
from . import print_test_ok


class TestIsOdd(unittest.TestCase):
    def valid_check(self, items):
        for item in items:
            try:
                self.assertTrue(is_odd(item))
            except Exception as e:
                print(f'failed for input: {item}')
                raise e

    def invalid_check(self, items):
        for item in items:
            try:
                self.assertFalse(is_odd(item))
            except Exception as e:
                print(f'failed for input: {item}')
                raise e

    def test_valid_odd(self):
        valid_items = [
            1,
            3,
            5,
            7,
            9,
            111,
            2019,
            -1
        ]
        self.valid_check(valid_items)
        print_test_ok()

    def test_invalid_odd(self):
        invalid_items = [
            ' ',
            'a',
            '.1',
            '1.1',
            0,
            2,
            110,
            220,
            2020,
            '2020',
            '666',
            '2',
            '0'
        ]
        self.invalid_check(invalid_items)
        print_test_ok()



import unittest

from pyvalidator.is_base58 import is_base58
from . import print_test_ok


class TestIsBase58(unittest.TestCase):

    def test_valid_base58(self):
        for i in [
            'BukQL',
            '3KMUV89zab',
            '91GHkLMNtyo98',
            'YyjKm3H',
            'Mkhss145TRFg',
            '7678765677',
            'abcodpq',
            'AAVHJKLPY',
        ]:
            self.assertTrue(is_base58(i))
        print_test_ok()

    def test_invalid_base58(self):
        for i in [
            '0OPLJH',
            'IMKLP23',
            'KLMOmk986',
            'LL1l1985hG',
            '*MP9K',
            ')()(=9292929MKL',
        ]:
            self.assertFalse(is_base58(i))
        print_test_ok()
